@@ -18,6 +18,7 @@ package com.google.errorprone;
 
 import static com.google.common.base.Preconditions.checkArgument;
 import static com.google.common.base.Preconditions.checkNotNull;
+import static com.google.errorprone.util.ASTHelpers.getStartPosition;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.ImmutableMap;
@@ -54,7 +55,7 @@ import com.sun.tools.javac.util.Name;
 import com.sun.tools.javac.util.Names;
 import com.sun.tools.javac.util.Options;
 import java.io.IOException;
-import java.lang.ref.WeakReference;
+import java.lang.ref.SoftReference;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -62,7 +63,9 @@ import java.util.Optional;
 import javax.annotation.Nullable;
 import javax.lang.model.util.Elements;
 
-/** @author alexeagle@google.com (Alex Eagle) */
+/**
+ * @author alexeagle@google.com (Alex Eagle)
+ */
 public class VisitorState {
 
   private final SharedState sharedState;
@@ -220,11 +223,6 @@ public class VisitorState {
     return new VisitorState(context, path, suppressedState, sharedState);
   }
 
-  @Deprecated // TODO(amalloy): Delete after next error-prone release.
-  public VisitorState withPathAndSuppression(TreePath path, SuppressedState suppressedState) {
-    return new VisitorState(context, path, suppressedState, sharedState);
-  }
-
   public VisitorState withSuppression(SuppressedState suppressedState) {
     if (suppressedState == this.suppressedState) {
       return this;
@@ -264,6 +262,10 @@ public class VisitorState {
     return sharedState.errorProneOptions;
   }
 
+  public Map<String, SeverityLevel> severityMap() {
+    return sharedState.severityMap;
+  }
+
   public void reportMatch(Description description) {
     checkNotNull(description, "Use Description.NO_MATCH to denote an absent finding.");
     if (description == Description.NO_MATCH) {
@@ -542,17 +544,21 @@ public class VisitorState {
    * This returns exactly what is in the source code, whereas .toString() pretty-prints the node
    * from its AST representation.
    *
-   * @return the source code that represents the node.
+   * @return the source code that represents the node, or {@code null} if the source code is
+   *     unavailable (e.g. for generated or desugared AST nodes)
    */
   @Nullable
   public String getSourceForNode(Tree tree) {
-    JCTree node = (JCTree) tree;
-    int start = node.getStartPosition();
-    int end = getEndPosition(node);
-    if (end < 0) {
+    int start = ((JCTree) tree).getStartPosition();
+    int end = getEndPosition(tree);
+    CharSequence source = getSourceCode();
+    if (end == -1) {
       return null;
     }
-    return getSourceCode().subSequence(start, end).toString();
+    checkArgument(start >= 0, "invalid start position (%s) for: %s", start, tree);
+    checkArgument(start < end, "invalid source positions (%s, %s) for: %s", start, end, tree);
+    checkArgument(end <= source.length(), "invalid end position (%s) for: %s", end, tree);
+    return source.subSequence(start, end).toString();
   }
 
   /**
@@ -573,7 +579,7 @@ public class VisitorState {
    * be used if a fix is already going to be emitted.
    */
   public List<ErrorProneToken> getOffsetTokensForNode(Tree tree) {
-    int start = ((JCTree) tree).getStartPosition();
+    int start = getStartPosition(tree);
     return ErrorProneTokens.getTokens(getSourceForNode(tree), start, context);
   }
 
@@ -660,7 +666,7 @@ public class VisitorState {
     (relying on the per-file cache in typeCache) if we don't have a result. If you want to cache a
     computation which can return null, wrap it in an Optional at the call site.*/
 
-    private WeakReference<T> cache = new WeakReference<>(null);
+    private SoftReference<T> cache = new SoftReference<>(null);
     private JavacInvocationInstance provenance;
 
     private Cache(Supplier<T> impl) {
@@ -677,14 +683,14 @@ public class VisitorState {
       if (value == null) {
         value = impl.get(state);
         if (value != null) {
-          cache = new WeakReference<>(value);
+          cache = new SoftReference<>(value);
           provenance = state.sharedState.javacInvocationInstance;
         }
       } else {
         JavacInvocationInstance current = state.sharedState.javacInvocationInstance;
         if (provenance != current) {
           value = impl.get(state);
-          cache = new WeakReference<>(value);
+          cache = new SoftReference<>(value);
           provenance = current;
         }
       }
@@ -746,4 +752,30 @@ public class VisitorState {
       this.errorProneOptions = errorProneOptions;
     }
   }
+
+  /**
+   * Returns the Java source code for a constant expression representing the given constant value.
+   * Like {@link Elements#getConstantExpression}, but doesn't over-escape single quotes in strings.
+   */
+  public String getConstantExpression(Object value) {
+    String escaped = getElements().getConstantExpression(value);
+    if (value instanceof String) {
+      // Don't escape single-quotes in string literals
+      StringBuilder sb = new StringBuilder();
+      for (int i = 0; i < escaped.length(); i++) {
+        char c = escaped.charAt(i);
+        if (c == '\\' && i + 1 < escaped.length()) {
+          char next = escaped.charAt(++i);
+          if (next != '\'') {
+            sb.append(c);
+          }
+          sb.append(next);
+        } else {
+          sb.append(c);
+        }
+      }
+      return sb.toString();
+    }
+    return escaped;
+  }
 }
