@@ -16,12 +16,18 @@
 
 package com.google.errorprone.bugpatterns;
 
-import static com.google.common.base.MoreObjects.firstNonNull;
 import static com.google.common.collect.ImmutableSet.toImmutableSet;
+import static com.google.errorprone.util.ASTHelpers.getModifiers;
+import static com.google.errorprone.util.ASTHelpers.getStartPosition;
+import static com.google.errorprone.util.ASTHelpers.getSymbol;
 
+import com.google.common.collect.ImmutableRangeSet;
 import com.google.common.collect.Iterables;
+import com.google.common.collect.Range;
 import com.google.errorprone.BugCheckerInfo;
 import com.google.errorprone.BugPattern.SeverityLevel;
+import com.google.errorprone.ErrorProneOptions;
+import com.google.errorprone.SuppressionInfo;
 import com.google.errorprone.VisitorState;
 import com.google.errorprone.annotations.CheckReturnValue;
 import com.google.errorprone.fixes.Fix;
@@ -82,6 +88,7 @@ import com.sun.source.tree.VariableTree;
 import com.sun.source.tree.WhileLoopTree;
 import com.sun.source.tree.WildcardTree;
 import com.sun.source.util.TreePathScanner;
+import com.sun.source.util.TreeScanner;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.tree.JCTree;
 import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
@@ -90,9 +97,7 @@ import java.io.Serializable;
 import java.lang.annotation.Annotation;
 import java.util.Arrays;
 import java.util.Collections;
-import java.util.Map;
 import java.util.Objects;
-import java.util.Optional;
 import java.util.Set;
 import java.util.function.BiPredicate;
 
@@ -142,32 +147,38 @@ public abstract class BugChecker implements Suppressible, Serializable {
 
   /** Helper to create a Description for the common case where there is a fix. */
   @CheckReturnValue
-  protected Description describeMatch(Tree node, Fix fix) {
+  public Description describeMatch(Tree node, Fix fix) {
     return buildDescription(node).addFix(fix).build();
   }
 
   /** Helper to create a Description for the common case where there is a fix. */
   @CheckReturnValue
-  protected Description describeMatch(JCTree node, Fix fix) {
+  public Description describeMatch(JCTree node, Fix fix) {
     return describeMatch((Tree) node, fix);
   }
 
   /** Helper to create a Description for the common case where there is a fix. */
   @CheckReturnValue
-  protected Description describeMatch(DiagnosticPosition position, Fix fix) {
+  public Description describeMatch(DiagnosticPosition position, Fix fix) {
     return buildDescription(position).addFix(fix).build();
   }
 
   /** Helper to create a Description for the common case where there is no fix. */
   @CheckReturnValue
-  protected Description describeMatch(Tree node) {
+  public Description describeMatch(Tree node) {
     return buildDescription(node).build();
   }
 
-  /** Helper to create a Description for the common case where there is an {@link Optional} fix. */
+  /** Helper to create a Description for the common case where there is no fix. */
   @CheckReturnValue
-  protected Description describeMatch(Tree node, Optional<? extends Fix> fix) {
-    return buildDescription(node).addFix(fix).build();
+  public Description describeMatch(JCTree node) {
+    return buildDescription(node).build();
+  }
+
+  /** Helper to create a Description for the common case where there is no fix. */
+  @CheckReturnValue
+  public Description describeMatch(DiagnosticPosition position) {
+    return buildDescription(position).build();
   }
 
   /**
@@ -216,10 +227,6 @@ public abstract class BugChecker implements Suppressible, Serializable {
     return info.defaultSeverity();
   }
 
-  public SeverityLevel severity(Map<String, SeverityLevel> severities) {
-    return firstNonNull(severities.get(canonicalName()), defaultSeverity());
-  }
-
   public String linkUrl() {
     return info.linkUrl();
   }
@@ -244,17 +251,17 @@ public abstract class BugChecker implements Suppressible, Serializable {
   }
 
   /**
-   * Returns true if the given tree is annotated with a {@code @SuppressWarnings} that disables this
-   * bug checker.
+   * @deprecated use {@link #isSuppressed(Tree, VisitorState)} instead
    */
+  @Deprecated
   public boolean isSuppressed(Tree tree) {
     return isSuppressed(ASTHelpers.getAnnotation(tree, SuppressWarnings.class));
   }
 
   /**
-   * Returns true if the given symbol is annotated with a {@code @SuppressWarnings} that disables
-   * this bug checker.
+   * @deprecated use {@link #isSuppressed(Symbol, VisitorState)} instead
    */
+  @Deprecated
   public boolean isSuppressed(Symbol symbol) {
     return isSuppressed(ASTHelpers.getAnnotation(symbol, SuppressWarnings.class));
   }
@@ -264,6 +271,55 @@ public abstract class BugChecker implements Suppressible, Serializable {
         && !Collections.disjoint(Arrays.asList(suppression.value()), allNames());
   }
 
+  /**
+   * Returns true if the given tree is annotated with a {@code @SuppressWarnings} that disables this
+   * bug checker.
+   */
+  public boolean isSuppressed(Tree tree, VisitorState state) {
+    Symbol sym = getSymbol(tree);
+    return sym != null && isSuppressed(sym, state);
+  }
+
+  /**
+   * Returns true if the given symbol is annotated with a {@code @SuppressWarnings} or other
+   * annotation that disables this bug checker.
+   */
+  public boolean isSuppressed(Symbol sym, VisitorState state) {
+    ErrorProneOptions errorProneOptions = state.errorProneOptions();
+    boolean suppressedInGeneratedCode =
+        errorProneOptions.disableWarningsInGeneratedCode()
+            && state.severityMap().get(canonicalName()) != SeverityLevel.ERROR;
+    SuppressionInfo.SuppressedState suppressedState =
+        SuppressionInfo.EMPTY
+            .withExtendedSuppressions(sym, state, customSuppressionAnnotationNames.get(state))
+            .suppressedState(BugChecker.this, suppressedInGeneratedCode, state);
+    return suppressedState == SuppressionInfo.SuppressedState.SUPPRESSED;
+  }
+
+  private final Supplier<? extends Set<? extends Name>> customSuppressionAnnotationNames =
+      VisitorState.memoize(
+          state ->
+              customSuppressionAnnotations().stream()
+                  .map(a -> state.getName(a.getName()))
+                  .collect(toImmutableSet()));
+
+  /** Computes a RangeSet of code regions which are suppressed by this bug checker. */
+  public ImmutableRangeSet<Integer> suppressedRegions(VisitorState state) {
+    ImmutableRangeSet.Builder<Integer> suppressedRegions = ImmutableRangeSet.builder();
+    new TreeScanner<Void, Void>() {
+      @Override
+      public Void scan(Tree tree, Void unused) {
+        if (getModifiers(tree) != null && isSuppressed(tree, state)) {
+          suppressedRegions.add(Range.closed(getStartPosition(tree), state.getEndPosition(tree)));
+        } else {
+          super.scan(tree, null);
+        }
+        return null;
+      }
+    }.scan(state.getPath().getCompilationUnit(), null);
+    return suppressedRegions.build();
+  }
+
   public interface AnnotationTreeMatcher extends Suppressible {
     Description matchAnnotation(AnnotationTree tree, VisitorState state);
   }
@@ -496,11 +552,34 @@ public abstract class BugChecker implements Suppressible, Serializable {
 
   /** A {@link TreePathScanner} which skips trees which are suppressed for this check. */
   protected class SuppressibleTreePathScanner<A, B> extends TreePathScanner<A, B> {
+
+    // TODO(cushon): make this protected once it is required; currently it would shadow
+    // other variables named state and break checks that pass the deprecated constructor
+    private final VisitorState state;
+
+    public SuppressibleTreePathScanner(VisitorState state) {
+      this.state = state;
+    }
+
+    /**
+     * @deprecated use {@link #SuppressibleTreePathScanner(VisitorState)} instead
+     */
+    @Deprecated
+    public SuppressibleTreePathScanner() {
+      this(null);
+    }
+
     @Override
-    public final A scan(Tree tree, B b) {
+    public A scan(Tree tree, B b) {
       boolean isSuppressible =
           tree instanceof ClassTree || tree instanceof MethodTree || tree instanceof VariableTree;
-      return isSuppressible && isSuppressed(tree) ? null : super.scan(tree, b);
+      if (isSuppressible) {
+        boolean suppressed = state != null ? isSuppressed(tree, state) : isSuppressed(tree);
+        if (suppressed) {
+          return null;
+        }
+      }
+      return super.scan(tree, b);
     }
   }
 }
