@@ -25,13 +25,11 @@ import static com.google.errorprone.dataflow.nullnesspropagation.Nullness.NULL;
 import static com.google.errorprone.dataflow.nullnesspropagation.Nullness.NULLABLE;
 import static com.sun.tools.javac.code.TypeTag.BOOLEAN;
 import static javax.lang.model.element.ElementKind.EXCEPTION_PARAMETER;
-import static org.checkerframework.shaded.javacutil.TreeUtils.elementFromDeclaration;
-import static org.checkerframework.shaded.javacutil.TreeUtils.enclosingOfClass;
+import static org.checkerframework.errorprone.javacutil.TreePathUtil.enclosingOfClass;
+import static org.checkerframework.errorprone.javacutil.TreeUtils.elementFromDeclaration;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
-import com.google.common.base.Predicate;
-import com.google.common.base.Predicates;
 import com.google.common.base.Strings;
 import com.google.common.base.Verify;
 import com.google.common.collect.ImmutableList;
@@ -43,7 +41,6 @@ import com.google.common.primitives.UnsignedLong;
 import com.google.errorprone.dataflow.AccessPath;
 import com.google.errorprone.dataflow.AccessPathStore;
 import com.google.errorprone.dataflow.AccessPathValues;
-import com.google.errorprone.dataflow.LocalVariableValues;
 import com.google.errorprone.dataflow.nullnesspropagation.inference.InferredNullability;
 import com.google.errorprone.dataflow.nullnesspropagation.inference.NullnessQualifierInference;
 import com.google.errorprone.util.MoreAnnotations;
@@ -86,26 +83,29 @@ import java.util.List;
 import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
+import java.util.function.Predicate;
 import javax.annotation.Nullable;
 import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeVariable;
-import org.checkerframework.shaded.dataflow.analysis.Analysis;
-import org.checkerframework.shaded.dataflow.cfg.CFGBuilder;
-import org.checkerframework.shaded.dataflow.cfg.ControlFlowGraph;
-import org.checkerframework.shaded.dataflow.cfg.UnderlyingAST;
-import org.checkerframework.shaded.dataflow.cfg.node.ArrayAccessNode;
-import org.checkerframework.shaded.dataflow.cfg.node.ArrayCreationNode;
-import org.checkerframework.shaded.dataflow.cfg.node.AssignmentNode;
-import org.checkerframework.shaded.dataflow.cfg.node.EqualToNode;
-import org.checkerframework.shaded.dataflow.cfg.node.FieldAccessNode;
-import org.checkerframework.shaded.dataflow.cfg.node.FunctionalInterfaceNode;
-import org.checkerframework.shaded.dataflow.cfg.node.InstanceOfNode;
-import org.checkerframework.shaded.dataflow.cfg.node.LocalVariableNode;
-import org.checkerframework.shaded.dataflow.cfg.node.MethodInvocationNode;
-import org.checkerframework.shaded.dataflow.cfg.node.Node;
-import org.checkerframework.shaded.dataflow.cfg.node.NotEqualNode;
-import org.checkerframework.shaded.dataflow.cfg.node.TypeCastNode;
-import org.checkerframework.shaded.dataflow.cfg.node.VariableDeclarationNode;
+import org.checkerframework.errorprone.dataflow.analysis.Analysis;
+import org.checkerframework.errorprone.dataflow.analysis.ForwardAnalysisImpl;
+import org.checkerframework.errorprone.dataflow.cfg.ControlFlowGraph;
+import org.checkerframework.errorprone.dataflow.cfg.UnderlyingAST;
+import org.checkerframework.errorprone.dataflow.cfg.builder.CFGBuilder;
+import org.checkerframework.errorprone.dataflow.cfg.node.ArrayAccessNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.ArrayCreationNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.AssignmentNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.EqualToNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.FieldAccessNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.FunctionalInterfaceNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.InstanceOfNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.LocalVariableNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.MethodInvocationNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.Node;
+import org.checkerframework.errorprone.dataflow.cfg.node.NotEqualNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.SwitchExpressionNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.TypeCastNode;
+import org.checkerframework.errorprone.dataflow.cfg.node.VariableDeclarationNode;
 
 /**
  * The {@code TransferFunction} for our nullability analysis. This analysis determines, for all
@@ -119,8 +119,8 @@ import org.checkerframework.shaded.dataflow.cfg.node.VariableDeclarationNode;
  * exception). For example, if {@code foo.toString()} is successfully evaluated, we know two things:
  *
  * <ol>
- *   <li>The expression itself is non-null (because {@code toString()} is in our whitelist of
- *       methods known to return non-null values)
+ *   <li>The expression itself is non-null (because {@code toString()} is in our list of methods
+ *       known to return non-null values)
  *   <li>{@code foo} is non-null (because it has been dereferenced without producing a {@code
  *       NullPointerException})
  * </ol>
@@ -199,7 +199,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
             String.class.getName());
 
     @Override
-    public boolean apply(MethodInfo methodInfo) {
+    public boolean test(MethodInfo methodInfo) {
       // Any method explicitly annotated is trusted to behave as advertised.
       Optional<Nullness> fromAnnotations =
           NullnessAnnotations.fromAnnotations(methodInfo.annotations());
@@ -307,7 +307,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
    * returning methods.
    */
   public NullnessPropagationTransfer(Predicate<MethodInfo> additionalNonNullReturningMethods) {
-    this(NULLABLE, Predicates.or(new ReturnValueIsNonNull(), additionalNonNullReturningMethods));
+    this(NULLABLE, new ReturnValueIsNonNull().or(additionalNonNullReturningMethods));
   }
 
   /**
@@ -353,7 +353,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
   // Literals
 
   @Override
-  Nullness visitThisLiteral() {
+  Nullness visitThis() {
     return NONNULL;
   }
 
@@ -406,7 +406,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
     ImmutableList<String> annotations =
         node.getType().getAnnotationMirrors().stream()
             .map(Object::toString)
-            .collect(ImmutableList.toImmutableList());
+            .collect(toImmutableList());
     return NullnessAnnotations.fromAnnotations(annotations)
         .orElseGet(
             () -> hasPrimitiveType(node) ? NONNULL : inputs.valueOfSubNode(node.getOperand()));
@@ -419,7 +419,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
    */
   @Override
   Nullness visitStringConcatenate() {
-    // TODO(user): Mark the inputs as dereferenced.
+    // TODO(b/158869538): Mark the inputs as dereferenced.
     return NONNULL;
   }
 
@@ -516,13 +516,13 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
    * <p>Edge case: {@code node} can be a captured local variable accessed from inside a local or
    * anonymous inner class, or possibly from inside a lambda expression (even though these manifest
    * as fields in bytecode). As of 7/2016 this analysis doesn't have any knowledge of captured local
-   * variables will essentially assume whatever default is used in {@link #values}.
+   * variables will essentially assume whatever default is used in {@code values}.
    */
   @Override
-  Nullness visitLocalVariable(LocalVariableNode node, LocalVariableValues<Nullness> values) {
+  Nullness visitLocalVariable(LocalVariableNode node, AccessPathValues<Nullness> values) {
     return hasPrimitiveType(node) || hasNonNullConstantValue(node)
         ? NONNULL
-        : values.valueOfLocalVariable(node, defaultAssumption);
+        : values.valueOfAccessPath(AccessPath.fromLocalVariable(node), defaultAssumption);
   }
 
   /**
@@ -583,6 +583,11 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
     return returnValueNullness(node, callee);
   }
 
+  @Override
+  Nullness visitSwitchExpression(SwitchExpressionNode node, SubNodeValues inputs, Updates updates) {
+    return NULLABLE;
+  }
+
   @Override
   Nullness visitObjectCreation() {
     return NONNULL;
@@ -759,7 +764,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
     }
 
     Optional<Nullness> declaredNullness = NullnessAnnotations.fromAnnotationsOn(accessed.symbol);
-    if (!declaredNullness.isPresent()) {
+    if (declaredNullness.isEmpty()) {
       Type ftype = accessed.symbol.type;
       if (ftype instanceof TypeVariable) {
         declaredNullness = NullnessAnnotations.getUpperBound((TypeVariable) ftype);
@@ -783,7 +788,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
       return NONNULL;
     }
 
-    Nullness assumedNullness = methodReturnsNonNull.apply(callee) ? NONNULL : NULLABLE;
+    Nullness assumedNullness = methodReturnsNonNull.test(callee) ? NONNULL : NULLABLE;
     if (!callee.isGenericResult) {
       // We only care about inference results for methods that return a type variable.
       return assumedNullness;
@@ -832,7 +837,7 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
               /*assumeAssertionsDisabled=*/ false,
               javacEnv);
       Analysis<Nullness, AccessPathStore<Nullness>, NullnessPropagationTransfer> analysis =
-          new Analysis<>(this);
+          new ForwardAnalysisImpl<>(this);
       analysis.performAnalysis(cfg);
       return analysis.getValue(initializerPath.getLeaf());
     } finally {
@@ -857,7 +862,8 @@ class NullnessPropagationTransfer extends AbstractNullnessPropagationTransfer
    */
   private static void setUnconditionalArgumentNullness(
       Updates bothUpdates, List<Node> arguments, ClassAndMethod callee) {
-    Set<Integer> requiredNonNullParameters = REQUIRED_NON_NULL_PARAMETERS.get(callee.name());
+    ImmutableSet<Integer> requiredNonNullParameters =
+        REQUIRED_NON_NULL_PARAMETERS.get(callee.name());
     for (LocalVariableNode var : variablesAtIndexes(requiredNonNullParameters, arguments)) {
       bothUpdates.set(var, NONNULL);
     }
