@@ -18,6 +18,7 @@ package com.google.errorprone.matchers;
 
 import static com.google.common.collect.ImmutableSet.toImmutableSet;
 import static com.google.common.collect.Iterables.getLast;
+import static com.google.common.collect.Iterables.getOnlyElement;
 import static com.google.common.collect.Streams.stream;
 import static com.google.errorprone.suppliers.Suppliers.BOOLEAN_TYPE;
 import static com.google.errorprone.suppliers.Suppliers.INT_TYPE;
@@ -62,6 +63,7 @@ import com.sun.source.tree.LiteralTree;
 import com.sun.source.tree.MemberSelectTree;
 import com.sun.source.tree.MethodInvocationTree;
 import com.sun.source.tree.MethodTree;
+import com.sun.source.tree.ReturnTree;
 import com.sun.source.tree.StatementTree;
 import com.sun.source.tree.SynchronizedTree;
 import com.sun.source.tree.Tree;
@@ -70,6 +72,7 @@ import com.sun.source.tree.TypeCastTree;
 import com.sun.source.tree.VariableTree;
 import com.sun.source.util.TreePath;
 import com.sun.tools.javac.code.Symbol;
+import com.sun.tools.javac.code.Symbol.ClassSymbol;
 import com.sun.tools.javac.code.Symbol.MethodSymbol;
 import com.sun.tools.javac.code.Symbol.VarSymbol;
 import com.sun.tools.javac.code.Type;
@@ -88,6 +91,7 @@ import java.util.List;
 import java.util.Objects;
 import java.util.Set;
 import java.util.function.BiPredicate;
+import java.util.function.Predicate;
 import java.util.regex.Pattern;
 import java.util.stream.Stream;
 import javax.lang.model.element.ElementKind;
@@ -127,7 +131,22 @@ public class Matchers {
    * given matchers do.
    */
   @SafeVarargs
-  public static <T extends Tree> Matcher<T> allOf(final Matcher<? super T>... matchers) {
+  public static <T extends Tree> Matcher<T> allOf(Matcher<? super T>... matchers) {
+    return (t, state) -> {
+      for (Matcher<? super T> matcher : matchers) {
+        if (!matcher.matches(t, state)) {
+          return false;
+        }
+      }
+      return true;
+    };
+  }
+
+  /**
+   * Compose several matchers together, such that the composite matches an AST node iff all the
+   * given matchers do.
+   */
+  public static <T extends Tree> Matcher<T> allOf(Iterable<? extends Matcher<? super T>> matchers) {
     return (t, state) -> {
       for (Matcher<? super T> matcher : matchers) {
         if (!matcher.matches(t, state)) {
@@ -142,8 +161,7 @@ public class Matchers {
    * Compose several matchers together, such that the composite matches an AST node if any of the
    * given matchers do.
    */
-  public static <T extends Tree> Matcher<T> anyOf(
-      final Iterable<? extends Matcher<? super T>> matchers) {
+  public static <T extends Tree> Matcher<T> anyOf(Iterable<? extends Matcher<? super T>> matchers) {
     return (t, state) -> {
       for (Matcher<? super T> matcher : matchers) {
         if (matcher.matches(t, state)) {
@@ -266,7 +284,7 @@ public class Matchers {
    *
    * @param argNum The number of the argument to compare against (zero-based.
    */
-  public static Matcher<? super MethodInvocationTree> receiverSameAsArgument(final int argNum) {
+  public static Matcher<? super MethodInvocationTree> receiverSameAsArgument(int argNum) {
     return (t, state) -> {
       List<? extends ExpressionTree> args = t.getArguments();
       if (args.size() <= argNum) {
@@ -288,7 +306,7 @@ public class Matchers {
   }
 
   public static Matcher<MethodInvocationTree> receiverOfInvocation(
-      final Matcher<ExpressionTree> expressionTreeMatcher) {
+      Matcher<ExpressionTree> expressionTreeMatcher) {
     return (methodInvocationTree, state) -> {
       ExpressionTree receiver = ASTHelpers.getReceiver(methodInvocationTree);
       return receiver != null && expressionTreeMatcher.matches(receiver, state);
@@ -314,14 +332,8 @@ public class Matchers {
     return new ConstructorOfClass(matchType, constructorMatcher);
   }
 
-  // TODO(cushon): expunge
-  public static Matcher<MethodInvocationTree> methodSelect(
-      Matcher<ExpressionTree> methodSelectMatcher) {
-    return new MethodInvocationMethodSelect(methodSelectMatcher);
-  }
-
   public static Matcher<MethodInvocationTree> argument(
-      final int position, final Matcher<ExpressionTree> argumentMatcher) {
+      int position, Matcher<ExpressionTree> argumentMatcher) {
     return new MethodInvocationArgument(position, argumentMatcher);
   }
 
@@ -350,7 +362,7 @@ public class Matchers {
    * methodSelectMatcher}. Ignores any arguments.
    */
   public static Matcher<ExpressionTree> methodInvocation(
-      final Matcher<ExpressionTree> methodSelectMatcher) {
+      Matcher<ExpressionTree> methodSelectMatcher) {
     return (expressionTree, state) -> {
       if (!(expressionTree instanceof MethodInvocationTree)) {
         return false;
@@ -360,7 +372,7 @@ public class Matchers {
     };
   }
 
-  public static Matcher<MethodInvocationTree> argumentCount(final int argumentCount) {
+  public static Matcher<MethodInvocationTree> argumentCount(int argumentCount) {
     return (t, state) -> t.getArguments().size() == argumentCount;
   }
 
@@ -369,7 +381,7 @@ public class Matchers {
    * parentNode(kindIs(Kind.RETURN))} would match the {@code this} expression in {@code return
    * this;}
    */
-  public static Matcher<Tree> parentNode(Matcher<Tree> treeMatcher) {
+  public static <T extends Tree> Matcher<T> parentNode(Matcher<Tree> treeMatcher) {
     return (tree, state) -> {
       TreePath parent = requireNonNull(state.getPath().getParentPath());
       return treeMatcher.matches(parent.getLeaf(), state.withPath(parent));
@@ -560,6 +572,13 @@ public class Matchers {
     };
   }
 
+  /** Matches an AST node if it is a null literal. */
+  public static Matcher<ExpressionTree> nullLiteral() {
+    return (tree, state) -> {
+      return tree.getKind() == Kind.NULL_LITERAL;
+    };
+  }
+
   /** Matches an AST node if it is a literal other than null. */
   public static Matcher<ExpressionTree> nonNullLiteral() {
     return (tree, state) -> {
@@ -641,8 +660,7 @@ public class Matchers {
     };
   }
 
-  public static Matcher<ExpressionTree> classLiteral(
-      final Matcher<? super ExpressionTree> classMatcher) {
+  public static Matcher<ExpressionTree> classLiteral(Matcher<? super ExpressionTree> classMatcher) {
     return (tree, state) -> {
       if (tree.getKind() == Kind.MEMBER_SELECT) {
         MemberSelectTree select = (MemberSelectTree) tree;
@@ -797,7 +815,7 @@ public class Matchers {
     };
   }
 
-  /** Matches a whitelisted method invocation that is known to never return null */
+  /** Matches a method invocation that is known to never return null. */
   public static Matcher<ExpressionTree> methodReturnsNonNull() {
     return anyOf(
         instanceMethod().onDescendantOf("java.lang.Object").named("toString"),
@@ -865,11 +883,15 @@ public class Matchers {
    * @param variableMatcher an array of matchers to apply to the parameters of the method
    */
   @SafeVarargs
-  public static Matcher<MethodTree> methodHasParameters(
-      final Matcher<VariableTree>... variableMatcher) {
+  public static Matcher<MethodTree> methodHasParameters(Matcher<VariableTree>... variableMatcher) {
     return methodHasParameters(ImmutableList.copyOf(variableMatcher));
   }
 
+  /** Matches an AST node that represents a method declaration with no parameters. */
+  public static Matcher<MethodTree> methodHasNoParameters() {
+    return methodHasParameters(ImmutableList.of());
+  }
+
   /**
    * Matches an AST node that represents a method declaration, based on the list of
    * variableMatchers. Applies the variableMatcher at index n to the parameter at index n and
@@ -881,7 +903,7 @@ public class Matchers {
    * @param variableMatcher a list of matchers to apply to the parameters of the method
    */
   public static Matcher<MethodTree> methodHasParameters(
-      final List<Matcher<VariableTree>> variableMatcher) {
+      List<Matcher<VariableTree>> variableMatcher) {
     return (methodTree, state) -> {
       if (methodTree.getParameters().size() != variableMatcher.size()) {
         return false;
@@ -971,6 +993,11 @@ public class Matchers {
     return (variableTree, state) -> ElementKind.FIELD == getSymbol(variableTree).getKind();
   }
 
+  /** Matches if a {@link ClassTree} is an enum declaration. */
+  public static Matcher<ClassTree> isEnum() {
+    return (classTree, state) -> getSymbol(classTree).getKind() == ElementKind.ENUM;
+  }
+
   /**
    * Matches an class based on whether it is nested in another class or method.
    *
@@ -1100,16 +1127,44 @@ public class Matchers {
     return (tree, state) -> ASTHelpers.sameVariable(tree, expr);
   }
 
-  /** Matches if the expression is provably non-null. */
-  public static Matcher<ExpressionTree> isNonNull() {
+  /**
+   * Matches if the expression is provably non-null.
+   *
+   * <p>Uses the Checker Framework dataflow implementation to determine nullness. Be careful about
+   * using this in performance-critical code.
+   */
+  public static Matcher<ExpressionTree> isNonNullUsingDataflow() {
     return new NullnessMatcher(Nullness.NONNULL);
   }
 
-  /** Matches if the expression is provably null. */
-  public static Matcher<ExpressionTree> isNull() {
+  /**
+   * @deprecated use {@link #isNonNullUsingDataflow} instead.
+   */
+  @Deprecated
+  public static Matcher<ExpressionTree> isNonNull() {
+    return isNonNullUsingDataflow();
+  }
+
+  /**
+   * Matches if the expression is provably null.
+   *
+   * <p>Uses the Checker Framework dataflow implementation to determine nullness. Be careful about
+   * using this in performance-critical code.
+   *
+   * <p>Prefer {@code kindIs(NULL_LITERAL)} unless you really need dataflow.
+   */
+  public static Matcher<ExpressionTree> isNullUsingDataflow() {
     return new NullnessMatcher(Nullness.NULL);
   }
 
+  /**
+   * @deprecated use {@link #isNullUsingDataflow} instead.
+   */
+  @Deprecated
+  public static Matcher<ExpressionTree> isNull() {
+    return isNullUsingDataflow();
+  }
+
   /**
    * Matches an enhanced for loop if all the given matchers match.
    *
@@ -1206,8 +1261,8 @@ public class Matchers {
    * @param treeMatcher The matcher to apply recursively to the tree.
    */
   public static <T extends Tree, V extends Tree> Matcher<T> contains(
-      Class<V> clazz, Matcher<V> treeMatcher) {
-    final Matcher<Tree> contains = new Contains(toType(clazz, treeMatcher));
+      Class<? extends V> clazz, Matcher<? super V> treeMatcher) {
+    Matcher<Tree> contains = new Contains(toType(clazz, treeMatcher));
     return contains::matches;
   }
 
@@ -1256,11 +1311,15 @@ public class Matchers {
   private static final ImmutableSet<Kind> DECLARATION =
       Sets.immutableEnumSet(Kind.LAMBDA_EXPRESSION, Kind.CLASS, Kind.ENUM, Kind.INTERFACE);
 
+  private static boolean isDeclaration(Kind kind) {
+    return DECLARATION.contains(kind) || kind.name().equals("RECORD");
+  }
+
   public static boolean methodCallInDeclarationOfThrowingRunnable(VisitorState state) {
     return stream(state.getPath())
         // Find the nearest definitional context for this method invocation
         // (i.e.: the nearest surrounding class or lambda)
-        .filter(t -> DECLARATION.contains(t.getKind()))
+        .filter(t -> isDeclaration(t.getKind()))
         .findFirst()
         .map(t -> isThrowingFunctionalInterface(getType(t), state))
         .orElseThrow(VerifyException::new);
@@ -1286,6 +1345,7 @@ public class Matchers {
                       "org.junit.jupiter.api.function.Executable",
                       "org.assertj.core.api.ThrowableAssert$ThrowingCallable",
                       "com.google.devtools.build.lib.testutil.MoreAsserts$ThrowingRunnable",
+                      "com.google.gerrit.testing.GerritJUnit$ThrowingRunnable",
                       "com.google.truth.ExpectFailure.AssertionCallback",
                       "com.google.truth.ExpectFailure.DelegatedAssertionCallback",
                       "com.google.truth.ExpectFailure.StandardSubjectBuilderCallback",
@@ -1294,7 +1354,6 @@ public class Matchers {
                   .filter(Objects::nonNull)
                   .collect(toImmutableSet()));
 
-
   private static class IsDirectImplementationOf extends ChildMultiMatcher<ClassTree, Tree> {
     public IsDirectImplementationOf(Matcher<Tree> classMatcher) {
       super(MatchType.AT_LEAST_ONE, classMatcher);
@@ -1306,15 +1365,18 @@ public class Matchers {
     }
   }
 
+  private static <T extends Tree> Matcher<T> packageMatches(Predicate<String> predicate) {
+    return (tree, state) -> predicate.test(getPackageFullName(state));
+  }
 
   /** Matches an AST node whose compilation unit's package name matches the given pattern. */
   public static <T extends Tree> Matcher<T> packageMatches(Pattern pattern) {
-    return (tree, state) -> pattern.matcher(getPackageFullName(state)).matches();
+    return packageMatches(pattern.asPredicate());
   }
 
   /** Matches an AST node whose compilation unit starts with this prefix. */
   public static <T extends Tree> Matcher<T> packageStartsWith(String prefix) {
-    return (tree, state) -> getPackageFullName(state).startsWith(prefix);
+    return packageMatches(s -> s.startsWith(prefix));
   }
 
   private static String getPackageFullName(VisitorState state) {
@@ -1322,44 +1384,65 @@ public class Matchers {
     return compilationUnit.packge.fullname.toString();
   }
 
-  private static final Matcher<MethodInvocationTree> STATIC_EQUALS =
+  private static final Matcher<ExpressionTree> STATIC_EQUALS =
       anyOf(
           allOf(
               staticMethod()
                   .onClass("android.support.v4.util.ObjectsCompat")
                   .named("equals")
                   .withParameters("java.lang.Object", "java.lang.Object"),
-              isSameType(BOOLEAN_TYPE)),
+              Matchers::methodReturnsBoolean),
+          allOf(
+              staticMethod()
+                  .onClass("androidx.v4.util.ObjectsCompat")
+                  .named("equals")
+                  .withParameters("java.lang.Object", "java.lang.Object"),
+              Matchers::methodReturnsBoolean),
           allOf(
               staticMethod()
                   .onClass("java.util.Objects")
                   .named("equals")
                   .withParameters("java.lang.Object", "java.lang.Object"),
-              isSameType(BOOLEAN_TYPE)),
+              Matchers::methodReturnsBoolean),
           allOf(
               staticMethod()
                   .onClass("com.google.common.base.Objects")
                   .named("equal")
                   .withParameters("java.lang.Object", "java.lang.Object"),
-              isSameType(BOOLEAN_TYPE)));
+              Matchers::methodReturnsBoolean));
 
   /**
    * Matches an invocation of a recognized static object equality method such as {@link
    * java.util.Objects#equals}. These are simple facades to {@link Object#equals} that accept null
    * for either argument.
    */
-  public static Matcher<MethodInvocationTree> staticEqualsInvocation() {
-    return STATIC_EQUALS;
+  @SuppressWarnings("unchecked") // safe covariant cast
+  public static <T extends ExpressionTree> Matcher<T> staticEqualsInvocation() {
+    return (Matcher<T>) STATIC_EQUALS;
   }
 
   private static final Matcher<ExpressionTree> INSTANCE_EQUALS =
       allOf(
           instanceMethod().anyClass().named("equals").withParameters("java.lang.Object"),
-          isSameType(BOOLEAN_TYPE));
+          Matchers::methodReturnsBoolean);
+
+  private static boolean methodReturnsBoolean(ExpressionTree tree, VisitorState state) {
+    return ASTHelpers.isSameType(
+        getSymbol(tree).type.getReturnType(), state.getSymtab().booleanType, state);
+  }
 
   /** Matches calls to the method {@link Object#equals(Object)} or any override of that method. */
-  public static Matcher<ExpressionTree> instanceEqualsInvocation() {
-    return INSTANCE_EQUALS;
+  @SuppressWarnings("unchecked") // safe covariant cast
+  public static <T extends ExpressionTree> Matcher<T> instanceEqualsInvocation() {
+    return (Matcher<T>) INSTANCE_EQUALS;
+  }
+
+  private static final Matcher<ExpressionTree> INSTANCE_HASHCODE =
+      allOf(instanceMethod().anyClass().named("hashCode").withNoParameters(), isSameType(INT_TYPE));
+
+  /** Matches calls to the method {@link Object#hashCode()} or any override of that method. */
+  public static Matcher<ExpressionTree> instanceHashCodeInvocation() {
+    return INSTANCE_HASHCODE;
   }
 
   private static final Matcher<ExpressionTree> ASSERT_EQUALS =
@@ -1404,7 +1487,7 @@ public class Matchers {
       allOf(
           methodIsNamed("toString"),
           methodHasVisibility(Visibility.PUBLIC),
-          methodHasParameters(),
+          methodHasNoParameters(),
           methodReturns(STRING_TYPE));
 
   /** Matches {@link Object#toString} method declaration. */
@@ -1416,7 +1499,7 @@ public class Matchers {
       allOf(
           methodIsNamed("hashCode"),
           methodHasVisibility(Visibility.PUBLIC),
-          methodHasParameters(),
+          methodHasNoParameters(),
           methodReturns(INT_TYPE));
 
   /** Matches {@code hashCode} method declaration. */
@@ -1424,6 +1507,19 @@ public class Matchers {
     return HASH_CODE_DECLARATION;
   }
 
+  /** Matcher for the overriding method of 'int java.lang.Comparable.compareTo(T other)' */
+  private static final Matcher<MethodTree> COMPARABLE_METHOD_MATCHER =
+      allOf(
+          methodIsNamed("compareTo"),
+          methodHasVisibility(Visibility.PUBLIC),
+          methodReturns(INT_TYPE),
+          methodHasArity(1));
+
+  /** Matches {@code compareTo} method declaration. */
+  public static Matcher<MethodTree> compareToMethodDeclaration() {
+    return COMPARABLE_METHOD_MATCHER;
+  }
+
   /** Method signature of serialization methods. */
   public static final Matcher<MethodTree> SERIALIZATION_METHODS =
       allOf(
@@ -1441,4 +1537,58 @@ public class Matchers {
               allOf(
                   methodIsNamed("writeReplace"),
                   methodReturns(typeFromString("java.lang.Object")))));
+
+  public static final Matcher<Tree> IS_INTERFACE =
+      (t, s) -> {
+        Symbol symbol = getSymbol(t);
+        return symbol instanceof ClassSymbol && symbol.isInterface();
+      };
+
+  /**
+   * Matches the {@code Tree} if it returns an expression matching {@code expressionTreeMatcher}.
+   */
+  public static Matcher<StatementTree> matchExpressionReturn(
+      Matcher<ExpressionTree> expressionTreeMatcher) {
+    return (statement, state) -> {
+      if (!(statement instanceof ReturnTree)) {
+        return false;
+      }
+      ExpressionTree expression = ((ReturnTree) statement).getExpression();
+      if (expression == null) {
+        return false;
+      }
+      return expressionTreeMatcher.matches(expression, state);
+    };
+  }
+
+  /**
+   * Matches a {@link BlockTree} if it single statement block with statement matching {@code
+   * statementMatcher}.
+   */
+  public static Matcher<BlockTree> matchSingleStatementBlock(
+      Matcher<StatementTree> statementMatcher) {
+    return (blockTree, state) -> {
+      if (blockTree == null) {
+        return false;
+      }
+      List<? extends StatementTree> statements = blockTree.getStatements();
+      if (statements.size() != 1) {
+        return false;
+      }
+      return statementMatcher.matches(getOnlyElement(statements), state);
+    };
+  }
+
+  /**
+   * Returns a matcher for {@link MethodTree} whose implementation contains a single return
+   * statement with expression matching the passed {@code expressionTreeMatcher}.
+   */
+  public static Matcher<MethodTree> singleStatementReturnMatcher(
+      Matcher<ExpressionTree> expressionTreeMatcher) {
+    Matcher<BlockTree> matcher =
+        matchSingleStatementBlock(matchExpressionReturn(expressionTreeMatcher));
+    return (methodTree, state) -> {
+      return matcher.matches(methodTree.getBody(), state);
+    };
+  }
 }
