@@ -32,6 +32,7 @@ import com.sun.source.tree.EnhancedForLoopTree;
 import com.sun.source.tree.ForLoopTree;
 import com.sun.source.tree.IdentifierTree;
 import com.sun.source.tree.ImportTree;
+import com.sun.source.tree.LambdaExpressionTree;
 import com.sun.source.tree.MemberSelectTree;
 import com.sun.source.tree.MethodInvocationTree;
 import com.sun.source.tree.MethodTree;
@@ -127,6 +128,10 @@ public final class FindIdentifiers {
 
   @Nullable
   private static ClassTree getEnclosingClass(TreePath treePath) {
+    if (treePath.getLeaf() instanceof ClassTree) {
+      return (ClassTree) treePath.getLeaf();
+    }
+
     while (treePath != null) {
       TreePath parent = treePath.getParentPath();
       if (parent == null) {
@@ -165,6 +170,11 @@ public final class FindIdentifiers {
             addIfVariable(stmt, result);
           }
           break;
+        case LAMBDA_EXPRESSION:
+          for (VariableTree param : ((LambdaExpressionTree) curr).getParameters()) {
+            result.add(ASTHelpers.getSymbol(param));
+          }
+          break;
         case METHOD:
           for (VariableTree param : ((MethodTree) curr).getParameters()) {
             result.add(ASTHelpers.getSymbol(param));
@@ -197,7 +207,7 @@ public final class FindIdentifiers {
           for (Type type : superTypes) {
             Scope scope = type.tsym.members();
             ImmutableList.Builder<VarSymbol> varsList = ImmutableList.builder();
-            for (Symbol var : scope.getSymbols(VarSymbol.class::isInstance)) {
+            for (Symbol var : ASTHelpers.scope(scope).getSymbols(VarSymbol.class::isInstance)) {
               varsList.add((VarSymbol) var);
             }
             result.addAll(varsList.build().reverse());
@@ -244,10 +254,12 @@ public final class FindIdentifiers {
                           ASTHelpers.getType(memberSelectTree.getExpression()),
                           /* skipInterface= */ false);
               for (Symbol var :
-                  scope.getSymbols(
-                      sym ->
-                          sym instanceof VarSymbol
-                              && sym.getSimpleName().equals(memberSelectTree.getIdentifier()))) {
+                  ASTHelpers.scope(scope)
+                      .getSymbols(
+                          sym ->
+                              sym instanceof VarSymbol
+                                  && sym.getSimpleName()
+                                      .equals(memberSelectTree.getIdentifier()))) {
                 result.add((VarSymbol) var);
               }
             }
@@ -324,7 +336,8 @@ public final class FindIdentifiers {
               if (scope == null) {
                 return ImmutableList.<VarSymbol>of().stream();
               }
-              return ImmutableList.copyOf(scope.getSymbols(VarSymbol.class::isInstance))
+              return ImmutableList.copyOf(
+                      ASTHelpers.scope(scope).getSymbols(VarSymbol.class::isInstance))
                   .reverse()
                   .stream()
                   .map(v -> (VarSymbol) v)
@@ -337,7 +350,7 @@ public final class FindIdentifiers {
    * Finds all identifiers in a tree. Takes an optional stop point as its argument: the depth-first
    * walk will stop if this node is encountered.
    */
-  private static final TreeScanner<Void, Void> createFindIdentifiersScanner(
+  private static TreeScanner<Void, Void> createFindIdentifiersScanner(
       ImmutableSet.Builder<Symbol> builder, @Nullable Tree stoppingPoint) {
     return new TreeScanner<Void, Void>() {
       @Override
@@ -371,7 +384,7 @@ public final class FindIdentifiers {
     };
   }
 
-  private static boolean isVisible(VarSymbol var, final TreePath path) {
+  private static boolean isVisible(VarSymbol var, TreePath path) {
     switch (var.getKind()) {
       case ENUM_CONSTANT:
       case FIELD:
@@ -392,11 +405,12 @@ public final class FindIdentifiers {
           // the enclosing static nested class (JLS 8.5.1).
           if (lowerThan(
               path,
+              (curr, unused) -> {
+                Symbol sym = ASTHelpers.getSymbol(curr);
+                return sym != null && sym.isStatic();
+              },
               (curr, unused) ->
-                  curr instanceof ClassTree && ASTHelpers.getSymbol((ClassTree) curr).isStatic(),
-              (curr, unused) ->
-                  curr instanceof ClassTree
-                      && ASTHelpers.getSymbol((ClassTree) curr).equals(var.owner))) {
+                  curr instanceof ClassTree && ASTHelpers.getSymbol(curr).equals(var.owner))) {
             return false;
           }
         }
@@ -447,13 +461,13 @@ public final class FindIdentifiers {
 
   /** Returns true iff the leaf node of the {@code path} occurs in a JLS 8.3.1 static context. */
   private static boolean inStaticContext(TreePath path) {
-    Tree prev = path.getLeaf();
-    path = path.getParentPath();
-
     ClassSymbol enclosingClass =
         ASTHelpers.getSymbol(ASTHelpers.findEnclosingNode(path, ClassTree.class));
     ClassSymbol directSuperClass = (ClassSymbol) enclosingClass.getSuperclass().tsym;
 
+    Tree prev = path.getLeaf();
+    path = path.getParentPath();
+
     for (Tree tree : path) {
       switch (tree.getKind()) {
         case METHOD:
