@@ -24,19 +24,22 @@ import static com.google.errorprone.matchers.Matchers.toType;
 import static com.google.errorprone.matchers.method.MethodMatchers.constructor;
 import static com.google.errorprone.matchers.method.MethodMatchers.instanceMethod;
 import static com.google.errorprone.matchers.method.MethodMatchers.staticMethod;
+import static com.google.errorprone.util.ASTHelpers.getStartPosition;
 
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Iterables;
+import com.google.common.io.Files;
 import com.google.errorprone.BugPattern;
 import com.google.errorprone.BugPattern.StandardTags;
+import com.google.errorprone.ErrorProneFlags;
 import com.google.errorprone.VisitorState;
 import com.google.errorprone.bugpatterns.BugChecker.MethodInvocationTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.NewClassTreeMatcher;
 import com.google.errorprone.fixes.Fix;
 import com.google.errorprone.fixes.SuggestedFix;
-import com.google.errorprone.fixes.SuggestedFix.Builder;
 import com.google.errorprone.matchers.Description;
 import com.google.errorprone.matchers.Matcher;
+import com.google.errorprone.suppliers.Supplier;
 import com.google.errorprone.suppliers.Suppliers;
 import com.google.errorprone.util.ASTHelpers;
 import com.sun.source.tree.AssignmentTree;
@@ -49,9 +52,9 @@ import com.sun.source.tree.VariableTree;
 import com.sun.source.util.TreeScanner;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.code.Type;
-import com.sun.tools.javac.tree.JCTree;
 import java.io.BufferedReader;
 import java.io.BufferedWriter;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileReader;
 import java.io.FileWriter;
@@ -70,7 +73,6 @@ import java.util.Scanner;
 
 /** A {@link BugChecker}; see the associated {@link BugPattern} annotation for details. */
 @BugPattern(
-    name = "DefaultCharset",
     summary =
         "Implicit use of the platform default charset, which can result in differing behaviour"
             + " between JVM executions or incorrect behavior if the encoding of the data source"
@@ -83,13 +85,13 @@ public class DefaultCharset extends BugChecker
   enum CharsetFix {
     UTF_8_FIX("UTF_8", "Specify UTF-8") {
       @Override
-      void addImport(SuggestedFix.Builder fix, VisitorState state) {
+      void addImport(SuggestedFix.Builder fix) {
         fix.addStaticImport("java.nio.charset.StandardCharsets.UTF_8");
       }
     },
     DEFAULT_CHARSET_FIX("Charset.defaultCharset()", "Specify default charset") {
       @Override
-      void addImport(SuggestedFix.Builder fix, VisitorState state) {
+      void addImport(SuggestedFix.Builder fix) {
         fix.addImport("java.nio.charset.Charset");
       }
     };
@@ -106,7 +108,7 @@ public class DefaultCharset extends BugChecker
       return replacement;
     }
 
-    abstract void addImport(SuggestedFix.Builder fix, VisitorState state);
+    abstract void addImport(SuggestedFix.Builder fix);
   }
 
   // ignore the constructor that takes FileDescriptor; it's rare and there's no automated fix
@@ -155,11 +157,17 @@ public class DefaultCharset extends BugChecker
       staticMethod().onClass("com.google.protobuf.ByteString").named("copyFrom");
 
   private static final Matcher<ExpressionTree> STRING_GET_BYTES =
-      instanceMethod().onExactClass(String.class.getName()).named("getBytes").withParameters();
+      instanceMethod().onExactClass(String.class.getName()).named("getBytes").withNoParameters();
+
+  private static final Matcher<ExpressionTree> BYTE_ARRAY_OUTPUT_STREAM_TO_STRING =
+      instanceMethod()
+          .onDescendantOf(ByteArrayOutputStream.class.getName())
+          .named("toString")
+          .withNoParameters();
 
   private static final Matcher<ExpressionTree> FILE_NEW_WRITER =
       staticMethod()
-          .onClass(com.google.common.io.Files.class.getName())
+          .onClass(Files.class.getName())
           .named("newWriter")
           .withParameters("java.lang.String");
 
@@ -190,42 +198,47 @@ public class DefaultCharset extends BugChecker
               .forClass(Scanner.class.getName())
               .withParameters(ReadableByteChannel.class.getName()));
 
+  private final boolean byteArrayOutputStreamToString;
+
+  public DefaultCharset(ErrorProneFlags flags) {
+    this.byteArrayOutputStreamToString =
+        flags.getBoolean("DefaultCharset:ByteArrayOutputStreamToString").orElse(true);
+  }
+
   @Override
   public Description matchMethodInvocation(MethodInvocationTree tree, VisitorState state) {
     if (state.isAndroidCompatible()) { // Android's default platform Charset is always UTF-8
       return NO_MATCH;
     }
     if (STRING_GET_BYTES.matches(tree, state)) {
-      Description.Builder description = buildDescription(tree);
       Tree parent = state.getPath().getParentPath().getLeaf();
       if (parent instanceof ExpressionTree
           && BYTESTRING_COPY_FROM.matches((ExpressionTree) parent, state)) {
-        byteStringFixes(description, tree, (ExpressionTree) parent, state);
+        return byteStringFixes(tree, (ExpressionTree) parent, state);
       } else {
-        appendCharsets(description, tree, tree.getMethodSelect(), tree.getArguments(), state);
+        return appendCharsets(tree, tree.getMethodSelect(), tree.getArguments(), state);
       }
-      return description.build();
     }
     if (FILE_NEW_WRITER.matches(tree, state)) {
-      Description.Builder description = buildDescription(tree);
-      appendCharsets(description, tree, tree.getMethodSelect(), tree.getArguments(), state);
-      return description.build();
+      return appendCharsets(tree, tree.getMethodSelect(), tree.getArguments(), state);
+    }
+    if (byteArrayOutputStreamToString && BYTE_ARRAY_OUTPUT_STREAM_TO_STRING.matches(tree, state)) {
+      return appendCharsets(tree, tree.getMethodSelect(), tree.getArguments(), state);
     }
     return NO_MATCH;
   }
 
-  private static void byteStringFixes(
-      Description.Builder description,
-      MethodInvocationTree tree,
-      ExpressionTree parent,
-      VisitorState state) {
-    description.addFix(byteStringFix(tree, parent, state, "copyFromUtf8(", "").build());
-
+  private Description byteStringFixes(
+      MethodInvocationTree tree, ExpressionTree parent, VisitorState state) {
     SuggestedFix.Builder builder =
         byteStringFix(
             tree, parent, state, "copyFrom(", ", " + CharsetFix.DEFAULT_CHARSET_FIX.replacement());
-    CharsetFix.DEFAULT_CHARSET_FIX.addImport(builder, state);
-    description.addFix(builder.build());
+    CharsetFix.DEFAULT_CHARSET_FIX.addImport(builder);
+
+    return buildDescription(tree)
+        .addFix(byteStringFix(tree, parent, state, "copyFromUtf8(", "").build())
+        .addFix(builder.build())
+        .build();
   }
 
   private static SuggestedFix.Builder byteStringFix(
@@ -235,16 +248,16 @@ public class DefaultCharset extends BugChecker
       String prefix,
       String suffix) {
     Tree parentReceiver = ASTHelpers.getReceiver(parent);
-    Builder fix = SuggestedFix.builder();
+    SuggestedFix.Builder fix = SuggestedFix.builder();
     if (parentReceiver != null) {
       fix.replace(
           /*startPos=*/ state.getEndPosition(parentReceiver),
-          /*endPos=*/ ((JCTree) tree).getStartPosition(),
+          /*endPos=*/ getStartPosition(tree),
           /*replaceWith=*/ "." + prefix);
     } else {
       fix.replace(
-          /*startPos=*/ ((JCTree) parent).getStartPosition(),
-          /*endPos=*/ ((JCTree) tree).getStartPosition(),
+          /*startPos=*/ getStartPosition(parent),
+          /*endPos=*/ getStartPosition(tree),
           /*replaceWith=*/ prefix);
     }
     fix.replace(
@@ -258,9 +271,7 @@ public class DefaultCharset extends BugChecker
       return NO_MATCH;
     }
     if (CTOR.matches(tree, state)) {
-      Description.Builder description = buildDescription(tree);
-      appendCharsets(description, tree, tree.getIdentifier(), tree.getArguments(), state);
-      return description.build();
+      return appendCharsets(tree, tree.getIdentifier(), tree.getArguments(), state);
     }
     if (FILE_READER.matches(tree, state)) {
       return handleFileReader(tree, state);
@@ -269,18 +280,18 @@ public class DefaultCharset extends BugChecker
       return handleFileWriter(tree, state);
     }
     if (PRINT_WRITER.matches(tree, state)) {
-      return handlePrintWriter(tree, state);
+      return handlePrintWriter(tree);
     }
     if (PRINT_WRITER_OUTPUTSTREAM.matches(tree, state)) {
-      return handlePrintWriterOutputStream(tree, state);
+      return handlePrintWriterOutputStream(tree);
     }
     if (SCANNER_MATCHER.matches(tree, state)) {
-      return handleScanner(tree, state);
+      return handleScanner(tree);
     }
     return NO_MATCH;
   }
 
-  private Description handleScanner(NewClassTree tree, VisitorState state) {
+  private Description handleScanner(NewClassTree tree) {
     Description.Builder description = buildDescription(tree);
     for (CharsetFix charsetFix : CharsetFix.values()) {
       SuggestedFix.Builder fix =
@@ -288,7 +299,7 @@ public class DefaultCharset extends BugChecker
               .postfixWith(
                   getOnlyElement(tree.getArguments()),
                   String.format(", %s.name()", charsetFix.replacement()));
-      charsetFix.addImport(fix, state);
+      charsetFix.addImport(fix);
       description.addFix(fix.build());
     }
     return description.build();
@@ -327,19 +338,20 @@ public class DefaultCharset extends BugChecker
     }
   }
 
-  private Fix nioFileReaderFix(VisitorState state, Tree arg, Tree toReplace, CharsetFix charset) {
+  private static Fix nioFileReaderFix(
+      VisitorState state, Tree arg, Tree toReplace, CharsetFix charset) {
     SuggestedFix.Builder fix = SuggestedFix.builder();
     fix.replace(
         toReplace,
         String.format(
             "Files.newBufferedReader(%s, %s)", toPath(state, arg, fix), charset.replacement()));
     fix.addImport("java.nio.file.Files");
-    charset.addImport(fix, state);
+    charset.addImport(fix);
     variableTypeFix(fix, state, FileReader.class, Reader.class);
     return fix.build();
   }
 
-  private Fix guavaFileReaderFix(
+  private static Fix guavaFileReaderFix(
       VisitorState state, Tree fileArg, Tree toReplace, CharsetFix charset) {
     SuggestedFix.Builder fix = SuggestedFix.builder();
     fix.replace(
@@ -347,12 +359,12 @@ public class DefaultCharset extends BugChecker
         String.format(
             "Files.newReader(%s, %s)", toFile(state, fileArg, fix), charset.replacement()));
     fix.addImport("com.google.common.io.Files");
-    charset.addImport(fix, state);
+    charset.addImport(fix);
     variableTypeFix(fix, state, FileReader.class, Reader.class);
     return fix.build();
   }
 
-  private void variableTypeFix(
+  private static void variableTypeFix(
       SuggestedFix.Builder fix, VisitorState state, Class<?> original, Class<?> replacement) {
     Tree parent = state.getPath().getParentPath().getLeaf();
     Symbol sym;
@@ -370,21 +382,21 @@ public class DefaultCharset extends BugChecker
         sym.type, state.getTypeFromString(original.getCanonicalName()), state)) {
       return;
     }
-    state
-        .getPath()
-        .getCompilationUnit()
-        .accept(
-            new TreeScanner<Void, Void>() {
-              @Override
-              public Void visitVariable(VariableTree node, Void unused) {
-                if (sym.equals(ASTHelpers.getSymbol(node))) {
-                  fix.replace(node.getType(), replacement.getSimpleName())
-                      .addImport(replacement.getCanonicalName());
-                }
-                return null;
-              }
-            },
-            null);
+    new TreeScanner<Void, Void>() {
+      @Override
+      public Void visitVariable(VariableTree node, Void unused) {
+        if (!sym.equals(ASTHelpers.getSymbol(node))) {
+          return null;
+        }
+        if (ASTHelpers.getStartPosition(node.getType()) == -1) {
+          // ignore synthetic tree nodes for `var`
+          return null;
+        }
+        fix.replace(node.getType(), replacement.getSimpleName())
+            .addImport(replacement.getCanonicalName());
+        return null;
+      }
+    }.scan(state.getPath().getCompilationUnit(), null);
   }
 
   private Description handleFileWriter(NewClassTree tree, VisitorState state) {
@@ -406,7 +418,7 @@ public class DefaultCharset extends BugChecker
     return description.build();
   }
 
-  private Fix guavaFileWriterFix(
+  private static Fix guavaFileWriterFix(
       VisitorState state, Tree fileArg, Tree toReplace, CharsetFix charset) {
     SuggestedFix.Builder fix = SuggestedFix.builder();
     fix.replace(
@@ -414,12 +426,12 @@ public class DefaultCharset extends BugChecker
         String.format(
             "Files.newWriter(%s, %s)", toFile(state, fileArg, fix), charset.replacement()));
     fix.addImport("com.google.common.io.Files");
-    charset.addImport(fix, state);
+    charset.addImport(fix);
     variableTypeFix(fix, state, FileWriter.class, Writer.class);
     return fix.build();
   }
 
-  private Fix nioFileWriterFix(
+  private static Fix nioFileWriterFix(
       VisitorState state,
       Tree appendTree,
       Tree fileArg,
@@ -437,7 +449,7 @@ public class DefaultCharset extends BugChecker
     sb.append(".newBufferedWriter(");
     sb.append(toPath(state, fileArg, fix));
     sb.append(", ").append(charset.replacement());
-    charset.addImport(fix, state);
+    charset.addImport(fix);
     if (appendTree != null) {
       sb.append(toAppendMode(fix, appendTree, state));
     }
@@ -448,7 +460,7 @@ public class DefaultCharset extends BugChecker
   }
 
   /** Convert a boolean append mode to a StandardOpenOption. */
-  private String toAppendMode(SuggestedFix.Builder fix, Tree appendArg, VisitorState state) {
+  private static String toAppendMode(SuggestedFix.Builder fix, Tree appendArg, VisitorState state) {
     // recognize constants to try to avoid `true ? CREATE, APPEND : CREATE`
     Boolean value = ASTHelpers.constValue(appendArg, Boolean.class);
     if (value != null) {
@@ -470,12 +482,12 @@ public class DefaultCharset extends BugChecker
   }
 
   /** Converts a {@code String} to a {@code File}. */
-  private Object toFile(VisitorState state, Tree fileArg, SuggestedFix.Builder fix) {
+  private static Object toFile(VisitorState state, Tree fileArg, SuggestedFix.Builder fix) {
     Type type = ASTHelpers.getType(fileArg);
     if (ASTHelpers.isSubtype(type, state.getSymtab().stringType, state)) {
       fix.addImport("java.io.File");
       return String.format("new File(%s)", state.getSourceForNode(fileArg));
-    } else if (ASTHelpers.isSubtype(type, state.getTypeFromString("java.io.File"), state)) {
+    } else if (ASTHelpers.isSubtype(type, JAVA_IO_FILE.get(state), state)) {
       return state.getSourceForNode(fileArg);
     } else {
       throw new AssertionError("unexpected type: " + type);
@@ -483,30 +495,27 @@ public class DefaultCharset extends BugChecker
   }
 
   /** Convert a {@code String} or {@code File} argument to a {@code Path}. */
-  private String toPath(VisitorState state, Tree fileArg, SuggestedFix.Builder fix) {
+  private static String toPath(VisitorState state, Tree fileArg, SuggestedFix.Builder fix) {
     Type type = ASTHelpers.getType(fileArg);
     if (ASTHelpers.isSubtype(type, state.getSymtab().stringType, state)) {
       fix.addImport("java.nio.file.Paths");
       return String.format("Paths.get(%s)", state.getSourceForNode(fileArg));
-    } else if (ASTHelpers.isSubtype(type, state.getTypeFromString("java.io.File"), state)) {
+    } else if (ASTHelpers.isSubtype(type, JAVA_IO_FILE.get(state), state)) {
       return String.format("%s.toPath()", state.getSourceForNode(fileArg));
     } else {
       throw new AssertionError("unexpected type: " + type);
     }
   }
 
-  private void appendCharsets(
-      Description.Builder description,
-      Tree tree,
-      Tree select,
-      List<? extends ExpressionTree> arguments,
-      VisitorState state) {
-    description.addFix(appendCharset(tree, select, arguments, state, CharsetFix.UTF_8_FIX));
-    description.addFix(
-        appendCharset(tree, select, arguments, state, CharsetFix.DEFAULT_CHARSET_FIX));
+  private Description appendCharsets(
+      Tree tree, Tree select, List<? extends ExpressionTree> arguments, VisitorState state) {
+    return buildDescription(tree)
+        .addFix(appendCharset(tree, select, arguments, state, CharsetFix.UTF_8_FIX))
+        .addFix(appendCharset(tree, select, arguments, state, CharsetFix.DEFAULT_CHARSET_FIX))
+        .build();
   }
 
-  private Fix appendCharset(
+  private static Fix appendCharset(
       Tree tree,
       Tree select,
       List<? extends ExpressionTree> arguments,
@@ -521,12 +530,12 @@ public class DefaultCharset extends BugChecker
     } else {
       fix.postfixWith(Iterables.getLast(arguments), ", " + charset.replacement());
     }
-    charset.addImport(fix, state);
+    charset.addImport(fix);
     fix.setShortDescription(charset.title);
     return fix.build();
   }
 
-  private Description handlePrintWriter(NewClassTree tree, VisitorState state) {
+  private Description handlePrintWriter(NewClassTree tree) {
     Description.Builder description = buildDescription(tree);
     for (CharsetFix charsetFix : CharsetFix.values()) {
       SuggestedFix.Builder fix =
@@ -534,13 +543,13 @@ public class DefaultCharset extends BugChecker
               .postfixWith(
                   getOnlyElement(tree.getArguments()),
                   String.format(", %s.name()", charsetFix.replacement()));
-      charsetFix.addImport(fix, state);
+      charsetFix.addImport(fix);
       description.addFix(fix.build());
     }
     return description.build();
   }
 
-  private Description handlePrintWriterOutputStream(NewClassTree tree, VisitorState state) {
+  private Description handlePrintWriterOutputStream(NewClassTree tree) {
     Tree outputStream = tree.getArguments().get(0);
     Description.Builder description = buildDescription(tree);
     for (CharsetFix charsetFix : CharsetFix.values()) {
@@ -548,11 +557,14 @@ public class DefaultCharset extends BugChecker
           SuggestedFix.builder()
               .prefixWith(outputStream, "new BufferedWriter(new OutputStreamWriter(")
               .postfixWith(outputStream, String.format(", %s))", charsetFix.replacement()));
-      charsetFix.addImport(fix, state);
+      charsetFix.addImport(fix);
       fix.addImport("java.io.BufferedWriter");
       fix.addImport("java.io.OutputStreamWriter");
       description.addFix(fix.build());
     }
     return description.build();
   }
+
+  private static final Supplier<Type> JAVA_IO_FILE =
+      VisitorState.memoize(state -> state.getTypeFromString("java.io.File"));
 }
