@@ -23,7 +23,6 @@ import static com.google.errorprone.matchers.Description.NO_MATCH;
 import static com.google.errorprone.matchers.Matchers.anyOf;
 import static com.google.errorprone.matchers.method.MethodMatchers.instanceMethod;
 import static com.google.errorprone.matchers.method.MethodMatchers.staticMethod;
-import static com.google.errorprone.util.ASTHelpers.getReceiverType;
 
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
@@ -31,7 +30,6 @@ import com.google.errorprone.BugPattern;
 import com.google.errorprone.VisitorState;
 import com.google.errorprone.bugpatterns.BugChecker.ClassTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.MemberReferenceTreeMatcher;
-import com.google.errorprone.bugpatterns.BugChecker.MethodInvocationTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.NewClassTreeMatcher;
 import com.google.errorprone.fixes.Fix;
 import com.google.errorprone.fixes.SuggestedFix;
@@ -62,19 +60,14 @@ import com.sun.tools.javac.code.Symbol.MethodSymbol;
 import com.sun.tools.javac.code.Symbol.VarSymbol;
 import com.sun.tools.javac.code.Type;
 import com.sun.tools.javac.code.Types;
+import com.sun.tools.javac.util.Position;
 import java.util.Optional;
 import javax.annotation.Nullable;
 
 /** A {@link BugChecker}; see the associated {@link BugPattern} annotation for details. */
-@BugPattern(
-    name = "JdkObsolete",
-    summary = "Suggests alternatives to obsolete JDK classes.",
-    severity = WARNING)
+@BugPattern(summary = "Suggests alternatives to obsolete JDK classes.", severity = WARNING)
 public class JdkObsolete extends BugChecker
-    implements NewClassTreeMatcher,
-        ClassTreeMatcher,
-        MemberReferenceTreeMatcher,
-        MethodInvocationTreeMatcher {
+    implements NewClassTreeMatcher, ClassTreeMatcher, MemberReferenceTreeMatcher {
 
   static class Obsolete {
     final String qualifiedName;
@@ -110,10 +103,6 @@ public class JdkObsolete extends BugChecker
                   return linkedListFix(tree, state);
                 }
               },
-              new Obsolete(
-                  "java.util.Date",
-                  "Date has a bad API that leads to bugs; prefer java.time.Instant or LocalDate."
-                  ),
               new Obsolete(
                   "java.util.Vector",
                   "Vector performs synchronization that is usually unnecessary; prefer ArrayList."),
@@ -172,9 +161,6 @@ public class JdkObsolete extends BugChecker
   @Override
   public Description matchNewClass(NewClassTree tree, VisitorState state) {
     MethodSymbol constructor = ASTHelpers.getSymbol(tree);
-    if (constructor == null) {
-      return NO_MATCH;
-    }
     Symbol owner = constructor.owner;
     Description description =
         describeIfObsolete(
@@ -213,9 +199,6 @@ public class JdkObsolete extends BugChecker
       return NO_MATCH;
     }
     ClassSymbol symbol = ASTHelpers.getSymbol(tree);
-    if (symbol == null) {
-      return NO_MATCH;
-    }
     return describeIfObsolete(null, state.getTypes().directSupertypes(symbol.asType()), state);
   }
 
@@ -228,18 +211,6 @@ public class JdkObsolete extends BugChecker
     return describeIfObsolete(tree.getQualifierExpression(), ImmutableList.of(type), state);
   }
 
-  @Override
-  public Description matchMethodInvocation(MethodInvocationTree tree, VisitorState state) {
-    if (MATCHER_STRINGBUFFER.matches(tree, state)) {
-      return NO_MATCH;
-    }
-    Type type = getReceiverType(tree.getMethodSelect());
-    if (type == null) {
-      return NO_MATCH;
-    }
-    return describeIfObsolete(tree, ImmutableList.of(type), state);
-  }
-
   private Description describeIfObsolete(
       @Nullable Tree tree, Iterable<Type> types, VisitorState state) {
     for (Type type : types) {
@@ -260,6 +231,7 @@ public class JdkObsolete extends BugChecker
     return NO_MATCH;
   }
 
+  @Nullable
   private static Type getMethodOrLambdaReturnType(VisitorState state) {
     for (Tree tree : state.getPath()) {
       switch (tree.getKind()) {
@@ -275,6 +247,7 @@ public class JdkObsolete extends BugChecker
     return null;
   }
 
+  @Nullable
   static Type getTargetType(VisitorState state) {
     Tree parent = state.getPath().getParentPath().getLeaf();
     Type type;
@@ -372,11 +345,13 @@ public class JdkObsolete extends BugChecker
     if (escape[0]) {
       return Optional.empty();
     }
-    return Optional.of(
-        SuggestedFix.builder()
-            .replace(newClassTree.getIdentifier(), "StringBuilder")
-            .replace(varTree.getType(), "StringBuilder")
-            .build());
+    SuggestedFix.Builder fix =
+        SuggestedFix.builder().replace(newClassTree.getIdentifier(), "StringBuilder");
+    if (ASTHelpers.getStartPosition(varTree.getType()) != Position.NOPOS) {
+      // If the variable is declared with `var`, there's no declaration type to change
+      fix = fix.replace(varTree.getType(), "StringBuilder");
+    }
+    return Optional.of(fix.build());
   }
 
   private static TreePath findEnclosingMethod(VisitorState state) {
@@ -441,9 +416,6 @@ public class JdkObsolete extends BugChecker
   private static boolean implementingObsoleteMethod(
       MethodTree enclosingMethod, VisitorState state, Type type) {
     MethodSymbol method = ASTHelpers.getSymbol(enclosingMethod);
-    if (method == null) {
-      return false;
-    }
     if (ASTHelpers.findSuperMethods(method, state.getTypes()).isEmpty()) {
       // not an override
       return false;
