@@ -56,6 +56,7 @@ import static com.sun.source.tree.Tree.Kind.RIGHT_SHIFT_ASSIGNMENT;
 import static com.sun.source.tree.Tree.Kind.UNSIGNED_RIGHT_SHIFT_ASSIGNMENT;
 import static com.sun.source.tree.Tree.Kind.XOR_ASSIGNMENT;
 
+import com.google.common.collect.ImmutableSet;
 import com.google.errorprone.BugPattern;
 import com.google.errorprone.VisitorState;
 import com.google.errorprone.annotations.NoAllocation;
@@ -74,6 +75,7 @@ import com.google.errorprone.bugpatterns.BugChecker.VariableTreeMatcher;
 import com.google.errorprone.fixes.SuggestedFix;
 import com.google.errorprone.matchers.Description;
 import com.google.errorprone.matchers.Matcher;
+import com.google.errorprone.matchers.Matchers;
 import com.google.errorprone.util.ASTHelpers;
 import com.sun.source.tree.AssignmentTree;
 import com.sun.source.tree.BinaryTree;
@@ -97,11 +99,9 @@ import com.sun.tools.javac.code.Symbol.VarSymbol;
 import com.sun.tools.javac.code.Type.ArrayType;
 import com.sun.tools.javac.tree.JCTree.JCExpression;
 import com.sun.tools.javac.tree.JCTree.JCMethodInvocation;
-import java.util.Collections;
 import java.util.EnumSet;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Set;
 
 /**
  * Checks methods annotated with {@code @NoAllocation} to ensure they really do not allocate.
@@ -162,8 +162,8 @@ public class NoAllocationChecker extends BugChecker
 
   private static final Matcher<ExpressionTree> primitiveArrayExpression = isPrimitiveArrayType();
 
-  private static final Set<Kind> ALL_COMPOUND_OPERATORS =
-      Collections.unmodifiableSet(
+  private static final ImmutableSet<Kind> ALL_COMPOUND_OPERATORS =
+      ImmutableSet.copyOf(
           EnumSet.of(
               AND_ASSIGNMENT,
               DIVIDE_ASSIGNMENT,
@@ -277,18 +277,12 @@ public class NoAllocationChecker extends BugChecker
           typeCast(not(isPrimitiveType()), primitiveExpression));
 
   /** Matches boxing by return. */
-  private static final Matcher<ReturnTree> boxingReturn =
-      new Matcher<ReturnTree>() {
-        @Override
-        public boolean matches(ReturnTree tree, VisitorState state) {
-          return allOf(
-                  not(withinThrowOrAnnotation),
-                  enclosingMethod(
-                      allOf(noAllocationMethodMatcher, methodReturnsNonPrimitiveType())),
-                  isPrimitiveType())
-              .matches(tree.getExpression(), state);
-        }
-      };
+  private static final Matcher<StatementTree> boxingReturn =
+      Matchers.matchExpressionReturn(
+          allOf(
+              not(withinThrowOrAnnotation),
+              enclosingMethod(allOf(noAllocationMethodMatcher, methodReturnsNonPrimitiveType())),
+              isPrimitiveType()));
 
   /** Matches boxing by method invocation, including varargs. */
   private static final Matcher<MethodInvocationTree> boxingInvocation =
@@ -399,9 +393,6 @@ public class NoAllocationChecker extends BugChecker
       return NO_MATCH;
     }
     MethodSymbol symbol = ASTHelpers.getSymbol(tree);
-    if (symbol == null) {
-      return NO_MATCH;
-    }
     return findSuperMethods(symbol, state.getTypes()).stream()
         .filter(s -> ASTHelpers.hasAnnotation(s, NoAllocation.class.getName(), state))
         .findAny()
