@@ -25,12 +25,14 @@ import static com.google.errorprone.matchers.Matchers.instanceMethod;
 import static com.google.errorprone.matchers.Matchers.receiverOfInvocation;
 import static com.google.errorprone.matchers.Matchers.staticMethod;
 import static com.google.errorprone.util.ASTHelpers.getReceiver;
+import static com.google.errorprone.util.ASTHelpers.getStartPosition;
 import static com.google.errorprone.util.ASTHelpers.getType;
 import static com.google.errorprone.util.ASTHelpers.isConsideredFinal;
 import static com.google.errorprone.util.ASTHelpers.isSameType;
 
-import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Iterables;
+import com.google.common.collect.Sets;
 import com.google.errorprone.BugPattern;
 import com.google.errorprone.ErrorProneFlags;
 import com.google.errorprone.VisitorState;
@@ -40,6 +42,7 @@ import com.google.errorprone.fixes.SuggestedFixes;
 import com.google.errorprone.matchers.Description;
 import com.google.errorprone.matchers.Matcher;
 import com.google.errorprone.matchers.Matchers;
+import com.google.errorprone.suppliers.Supplier;
 import com.google.errorprone.util.ASTHelpers;
 import com.sun.source.tree.BinaryTree;
 import com.sun.source.tree.ClassTree;
@@ -57,10 +60,8 @@ import com.sun.source.util.TreePathScanner;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.code.Symbol.MethodSymbol;
 import com.sun.tools.javac.code.Type;
-import com.sun.tools.javac.tree.JCTree;
 import com.sun.tools.javac.tree.JCTree.JCFieldAccess;
 import java.util.Arrays;
-import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Objects;
@@ -69,16 +70,9 @@ import java.util.Set;
 import javax.annotation.Nullable;
 
 /** Matches comparison of proto fields to {@code null}. */
-@BugPattern(
-    name = "ProtoFieldNullComparison",
-    summary = "Protobuf fields cannot be null.",
-    severity = ERROR)
+@BugPattern(summary = "Protobuf fields cannot be null.", severity = ERROR)
 public class ProtoFieldNullComparison extends BugChecker implements CompilationUnitTreeMatcher {
 
-  private static final String PROTO_SUPER_CLASS = "com.google.protobuf.GeneratedMessage";
-
-  private static final String PROTO_LITE_SUPER_CLASS = "com.google.protobuf.GeneratedMessageLite";
-
   // TODO(b/111109484): Try to consolidate these with NullnessPropagationTransfer.
   private static final Matcher<ExpressionTree> CHECK_NOT_NULL =
       anyOf(
@@ -93,18 +87,18 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
 
   private static final Matcher<MethodInvocationTree> TRUTH_NOT_NULL =
       allOf(
-          instanceMethod().onDescendantOf("com.google.common.truth.Subject").named("isNotNull"),
+          instanceMethod().anyClass().named("isNotNull"),
           receiverOfInvocation(
               anyOf(
-                  staticMethod().onClass("com.google.common.truth.Truth").namedAnyOf("assertThat"),
+                  staticMethod().anyClass().namedAnyOf("assertThat"),
                   instanceMethod()
                       .onDescendantOf("com.google.common.truth.StandardSubjectBuilder")
                       .named("that"))));
 
   private static final Matcher<Tree> RETURNS_LIST = Matchers.isSubtypeOf("java.util.List");
 
-  private static final Set<Kind> COMPARISON_OPERATORS =
-      EnumSet.of(Kind.EQUAL_TO, Kind.NOT_EQUAL_TO);
+  private static final ImmutableSet<Kind> COMPARISON_OPERATORS =
+      Sets.immutableEnumSet(Kind.EQUAL_TO, Kind.NOT_EQUAL_TO);
 
   private static final Matcher<ExpressionTree> EXTENSION_METHODS_WITH_FIX =
       instanceMethod()
@@ -138,33 +132,15 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
 
   /** Matcher for generated protobufs. */
   private static final Matcher<ExpressionTree> PROTO_RECEIVER =
-      instanceMethod().onDescendantOfAny(PROTO_SUPER_CLASS, PROTO_LITE_SUPER_CLASS);
+      instanceMethod()
+          .onDescendantOfAny(
+              "com.google.protobuf.GeneratedMessageLite", "com.google.protobuf.GeneratedMessage");
 
-  /** Only track assignments of variables matching this. */
-  private final Matcher<ExpressionTree> trackAssignments;
-
-  private final boolean matchListGetters;
   private final boolean matchTestAssertions;
-  private final boolean descendIntoInitializers;
-  private final boolean matchOptionals;
 
   public ProtoFieldNullComparison(ErrorProneFlags flags) {
-    boolean trackServerProtoAssignments =
-        flags.getBoolean("ProtoFieldNullComparison:TrackServerProtoAssignments").orElse(true);
-    this.matchListGetters =
-        flags.getBoolean("ProtoFieldNullComparison:MatchListGetters").orElse(true);
     this.matchTestAssertions =
         flags.getBoolean("ProtoFieldNullComparison:MatchTestAssertions").orElse(false);
-    this.descendIntoInitializers =
-        flags.getBoolean("ProtoFieldNullComparison:DescendIntoInitializers").orElse(true);
-    this.matchOptionals = flags.getBoolean("ProtoFieldNullComparison:MatchOptionals").orElse(true);
-
-    ImmutableList.Builder<String> toTrack =
-        ImmutableList.<String>builder().add(PROTO_LITE_SUPER_CLASS);
-    if (trackServerProtoAssignments) {
-      toTrack.add(PROTO_SUPER_CLASS);
-    }
-    trackAssignments = instanceMethod().onDescendantOfAny(toTrack.build());
   }
 
   @Override
@@ -184,28 +160,22 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
 
     @Override
     public Void visitMethod(MethodTree method, Void unused) {
-      return isSuppressed(method) ? null : super.visitMethod(method, unused);
+      return isSuppressed(method, state) ? null : super.visitMethod(method, unused);
     }
 
     @Override
     public Void visitClass(ClassTree clazz, Void unused) {
-      return isSuppressed(clazz) ? null : super.visitClass(clazz, unused);
+      return isSuppressed(clazz, state) ? null : super.visitClass(clazz, unused);
     }
 
     @Override
     public Void visitVariable(VariableTree variable, Void unused) {
       Symbol symbol = ASTHelpers.getSymbol(variable);
-      if (variable.getInitializer() != null && symbol != null && isConsideredFinal(symbol)) {
-        if (descendIntoInitializers) {
-          getInitializer(variable.getInitializer())
-              .ifPresent(e -> effectivelyFinalValues.put(symbol, e));
-        } else {
-          if (trackAssignments.matches(variable.getInitializer(), state)) {
-            effectivelyFinalValues.put(symbol, variable.getInitializer());
-          }
-        }
+      if (variable.getInitializer() != null && isConsideredFinal(symbol)) {
+        getInitializer(variable.getInitializer())
+            .ifPresent(e -> effectivelyFinalValues.put(symbol, e));
       }
-      return isSuppressed(variable) ? null : super.visitVariable(variable, null);
+      return isSuppressed(variable, state) ? null : super.visitVariable(variable, null);
     }
 
     private Optional<ExpressionTree> getInitializer(ExpressionTree tree) {
@@ -213,7 +183,7 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
           new SimpleTreeVisitor<ExpressionTree, Void>() {
             @Override
             public ExpressionTree visitMethodInvocation(MethodInvocationTree node, Void unused) {
-              return trackAssignments.matches(node, state) ? node : null;
+              return PROTO_RECEIVER.matches(node, state) ? node : null;
             }
 
             @Override
@@ -258,7 +228,7 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
       } else if (matchTestAssertions && ASSERT_NOT_NULL.matches(node, subState)) {
         argument = getLast(node.getArguments());
         problemType = ProblemUsage.JUNIT;
-      } else if (matchOptionals && OF_NULLABLE.matches(node, subState)) {
+      } else if (OF_NULLABLE.matches(node, subState)) {
         argument = getOnlyElement(node.getArguments());
         problemType = ProblemUsage.OPTIONAL;
       } else if (matchTestAssertions && TRUTH_NOT_NULL.matches(node, subState)) {
@@ -268,7 +238,9 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
         return super.visitMethodInvocation(node, null);
       }
       getFixer(argument, subState)
-          .map(f -> describeMatch(node, problemType.fix(f, node, subState)))
+          .map(f -> problemType.fix(f, node, subState))
+          .filter(f -> !f.isEmpty())
+          .map(f -> describeMatch(node, f))
           .ifPresent(state::reportMatch);
 
       return super.visitMethodInvocation(node, null);
@@ -280,7 +252,6 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
         return Optional.empty();
       }
       return Arrays.stream(GetterTypes.values())
-          .filter(g -> matchListGetters || g != GetterTypes.VECTOR_INDEXED)
           .map(type -> type.match(resolvedTree, state))
           .filter(Objects::nonNull)
           .findFirst();
@@ -310,7 +281,9 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
   /** Generates a replacement hazzer, if available. */
   @FunctionalInterface
   private interface Fixer {
-    /** @param negated whether the hazzer should be negated. */
+    /**
+     * @param negated whether the hazzer should be negated.
+     */
     Optional<String> getHazzer(boolean negated, VisitorState state);
   }
 
@@ -349,7 +322,8 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
         if (hasMethods.isEmpty()) {
           return Optional.empty();
         }
-        String replacement = replaceLast(methodInvocation.toString(), methodName, hasMethod);
+        String replacement =
+            replaceLast(state.getSourceForNode(methodInvocation), methodName, hasMethod);
         return Optional.of(negated ? ("!" + replacement) : replacement);
       }
 
@@ -374,18 +348,19 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
             getType(getOnlyElement(method.getArguments())), state.getSymtab().intType, state)) {
           return null;
         }
-        return (n, s) -> Optional.of(generateFix(method, n));
+        return (n, s) -> Optional.of(generateFix(method, n, state));
       }
 
-      private String generateFix(MethodInvocationTree methodInvocation, boolean negated) {
+      private String generateFix(
+          MethodInvocationTree methodInvocation, boolean negated, VisitorState visitorState) {
         String methodName = ASTHelpers.getSymbol(methodInvocation).getQualifiedName().toString();
         String countMethod = methodName + "Count";
         return String.format(
             "%s.%s() %s %s",
-            getReceiver(methodInvocation),
+            visitorState.getSourceForNode(getReceiver(methodInvocation)),
             countMethod,
             negated ? "<=" : ">",
-            getOnlyElement(methodInvocation.getArguments()));
+            visitorState.getSourceForNode(getOnlyElement(methodInvocation.getArguments())));
       }
     },
     /** {@code proto.getRepeatedFooList()} */
@@ -403,11 +378,14 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
           return null;
         }
         ExpressionTree expressionTree = method.getMethodSelect();
-        return isGetter(expressionTree) ? (n, s) -> Optional.of(generateFix(n, method)) : null;
+        return isGetter(expressionTree)
+            ? (n, s) -> Optional.of(generateFix(n, method, state))
+            : null;
       }
 
-      private String generateFix(boolean negated, ExpressionTree methodInvocation) {
-        String replacement = methodInvocation + ".isEmpty()";
+      private String generateFix(
+          boolean negated, ExpressionTree methodInvocation, VisitorState state) {
+        String replacement = state.getSourceForNode(methodInvocation) + ".isEmpty()";
         return negated ? replacement : ("!" + replacement);
       }
     },
@@ -425,7 +403,7 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
           MethodInvocationTree methodInvocation = (MethodInvocationTree) tree;
           Type argumentType =
               ASTHelpers.getType(Iterables.getOnlyElement(methodInvocation.getArguments()));
-          Symbol extension = state.getSymbolFromString("com.google.protobuf.ExtensionLite");
+          Symbol extension = COM_GOOGLE_PROTOBUF_EXTENSIONLITE.get(state);
           Type genericsArgument = state.getTypes().asSuper(argumentType, extension);
 
           // If there are not two arguments then it is a raw type
@@ -438,9 +416,7 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
           // If the second element within the generic argument is a subtype of list,
           // that means it is a repeated field and therefore we cannot make a fix.
           if (ASTHelpers.isSubtype(
-              genericsArgument.getTypeArguments().get(1),
-              state.getTypeFromString("java.util.List"),
-              state)) {
+              genericsArgument.getTypeArguments().get(1), state.getSymtab().listType, state)) {
             return GetterTypes::emptyFix;
           }
           // Now that it is guaranteed that there is not a repeated field, providing a fix is safe
@@ -453,7 +429,8 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
         return (negated, state) -> {
           String methodName = getMethodName(methodInvocation);
           String hasMethod = methodName.replaceFirst("get", "has");
-          String replacement = replaceLast(methodInvocation.toString(), methodName, hasMethod);
+          String replacement =
+              replaceLast(state.getSourceForNode(methodInvocation), methodName, hasMethod);
           return Optional.of(negated ? "!" + replacement : replacement);
         };
       }
@@ -485,9 +462,7 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
       @Override
       SuggestedFix fix(Fixer fixer, ExpressionTree tree, VisitorState state) {
         Optional<String> replacement = fixer.getHazzer(tree.getKind() == Kind.EQUAL_TO, state);
-        return replacement
-            .map(r -> SuggestedFix.replace(tree, r))
-            .orElse(SuggestedFix.builder().build());
+        return replacement.map(r -> SuggestedFix.replace(tree, r)).orElse(SuggestedFix.emptyFix());
       }
     },
     /** Matches comparisons with Truth, i.e. {@code assertThat(proto.getField()).isNull()}. */
@@ -505,7 +480,7 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
                           "%s(%s).isTrue()",
                           state.getSourceForNode(receiver.getMethodSelect()), r));
                 })
-            .orElse(SuggestedFix.builder().build());
+            .orElse(SuggestedFix.emptyFix());
       }
     },
     /** Matches comparisons with JUnit, i.e. {@code assertNotNull(proto.getField())}. */
@@ -517,13 +492,13 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
             .getHazzer(/* negated= */ false, state)
             .map(
                 r -> {
-                  int startPos = ((JCTree) methodInvocationTree).getStartPosition();
+                  int startPos = getStartPosition(methodInvocationTree);
                   return SuggestedFix.builder()
                       .replace(getLast(methodInvocationTree.getArguments()), r)
                       .replace(startPos, startPos + "assertNotNull".length(), "assertTrue")
                       .build();
                 })
-            .orElse(SuggestedFix.builder().build());
+            .orElse(SuggestedFix.emptyFix());
       }
     },
     /** Matches precondition checks, i.e. {@code checkNotNull(proto.getField())}. */
@@ -549,4 +524,7 @@ public class ProtoFieldNullComparison extends BugChecker implements CompilationU
 
     abstract SuggestedFix fix(Fixer fixer, ExpressionTree tree, VisitorState state);
   }
+
+  private static final Supplier<Symbol> COM_GOOGLE_PROTOBUF_EXTENSIONLITE =
+      VisitorState.memoize(state -> state.getSymbolFromString("com.google.protobuf.ExtensionLite"));
 }
