@@ -24,9 +24,12 @@ import static com.google.common.collect.Iterables.getLast;
 import static com.google.common.collect.Iterables.getOnlyElement;
 import static com.google.errorprone.BugPattern.SeverityLevel.WARNING;
 import static com.google.errorprone.matchers.Matchers.SERIALIZATION_METHODS;
+import static com.google.errorprone.util.ASTHelpers.canBeRemoved;
+import static com.google.errorprone.util.ASTHelpers.getStartPosition;
 import static com.google.errorprone.util.ASTHelpers.getSymbol;
 import static com.google.errorprone.util.ASTHelpers.getType;
 import static com.google.errorprone.util.ASTHelpers.isSubtype;
+import static com.google.errorprone.util.ASTHelpers.shouldKeep;
 import static com.google.errorprone.util.SideEffectAnalysis.hasSideEffect;
 import static com.sun.source.tree.Tree.Kind.POSTFIX_DECREMENT;
 import static com.sun.source.tree.Tree.Kind.POSTFIX_INCREMENT;
@@ -43,6 +46,7 @@ import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.ListMultimap;
 import com.google.common.collect.Multimaps;
+import com.google.common.collect.Streams;
 import com.google.errorprone.BugPattern;
 import com.google.errorprone.ErrorProneFlags;
 import com.google.errorprone.VisitorState;
@@ -50,6 +54,7 @@ import com.google.errorprone.bugpatterns.BugChecker.CompilationUnitTreeMatcher;
 import com.google.errorprone.fixes.SuggestedFix;
 import com.google.errorprone.fixes.SuggestedFixes;
 import com.google.errorprone.matchers.Description;
+import com.google.errorprone.suppliers.Supplier;
 import com.google.errorprone.suppliers.Suppliers;
 import com.google.errorprone.util.ASTHelpers;
 import com.sun.source.tree.AnnotationTree;
@@ -109,7 +114,6 @@ import javax.lang.model.type.NullType;
 
 /** Bugpattern to detect unused declarations. */
 @BugPattern(
-    name = "UnusedVariable",
     altNames = {"unused", "UnusedParameters"},
     summary = "Unused.",
     severity = WARNING,
@@ -119,7 +123,6 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
 
   private static final ImmutableSet<String> EXEMPT_NAMES = ImmutableSet.of("ignored");
 
-
   /**
    * The set of annotation full names which exempt annotated element from being reported as unused.
    */
@@ -131,23 +134,23 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
           "javax.persistence.Version",
           "javax.xml.bind.annotation.XmlElement",
           "org.junit.Rule",
-          "org.mockito.Mock",
+          "org.openqa.selenium.support.FindAll",
           "org.openqa.selenium.support.FindBy",
-          "org.openqa.selenium.support.FindBys");
+          "org.openqa.selenium.support.FindBys",
+          "org.apache.beam.sdk.transforms.DoFn.TimerId",
+          "org.apache.beam.sdk.transforms.DoFn.StateId");
 
   private final ImmutableSet<String> methodAnnotationsExemptingParameters;
 
   /** The set of types exempting a type that is extending or implementing them. */
-  private static final ImmutableSet<String> EXEMPTING_SUPER_TYPES =
-      ImmutableSet.of(
-          );
+  private static final ImmutableSet<String> EXEMPTING_SUPER_TYPES = ImmutableSet.of();
 
   /** The set of types exempting a field of type extending them. */
   private static final ImmutableSet<String> EXEMPTING_FIELD_SUPER_TYPES =
       ImmutableSet.of("org.junit.rules.TestRule");
 
-  private static final ImmutableList<String> SPECIAL_FIELDS =
-      ImmutableList.of(
+  private static final ImmutableSet<String> SPECIAL_FIELDS =
+      ImmutableSet.of(
           "serialVersionUID",
           // TAG fields are used by convention in Android apps.
           "TAG");
@@ -156,8 +159,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
 
   public UnusedVariable(ErrorProneFlags flags) {
     ImmutableSet.Builder<String> methodAnnotationsExemptingParameters =
-        ImmutableSet.<String>builder()
-            .add("org.robolectric.annotation.Implementation");
+        ImmutableSet.<String>builder().add("org.robolectric.annotation.Implementation");
     flags
         .getList("Unused:methodAnnotationsExemptingParameters")
         .ifPresent(methodAnnotationsExemptingParameters::addAll);
@@ -260,25 +262,35 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       ImmutableList<TreePath> allUsageSites,
       VisitorState state) {
     if (unusedSymbol.getKind() != ElementKind.LOCAL_VARIABLE) {
-      return SuggestedFix.builder().build();
+      return SuggestedFix.emptyFix();
     }
     Optional<VariableTree> removedVariableTree =
         allUsageSites.stream()
             .filter(tp -> tp.getLeaf() instanceof VariableTree)
             .findFirst()
             .map(tp -> (VariableTree) tp.getLeaf());
+
+    // Find the first reassignment which wasn't only used by an ultimately unused assignment. If
+    // there is one, it should become a variable declaration.
     Optional<AssignmentTree> reassignment =
         specs.stream()
             .map(UnusedSpec::terminatingAssignment)
-            .filter(Optional::isPresent)
-            .map(Optional::get)
-            .filter(a -> allUsageSites.stream().noneMatch(tp -> tp.getLeaf().equals(a)))
+            .flatMap(Streams::stream)
+            .filter(
+                a ->
+                    allUsageSites.stream()
+                        .noneMatch(
+                            tp ->
+                                tp.getLeaf() instanceof ExpressionStatementTree
+                                    && ((ExpressionStatementTree) tp.getLeaf())
+                                        .getExpression()
+                                        .equals(a)))
             .findFirst();
-    if (!removedVariableTree.isPresent() || !reassignment.isPresent()) {
-      return SuggestedFix.builder().build();
+    if (removedVariableTree.isPresent() && reassignment.isPresent()) {
+      return SuggestedFix.prefixWith( // not needed if top-level statement
+          reassignment.get(), state.getSourceForNode(removedVariableTree.get().getType()) + " ");
     }
-    return SuggestedFix.prefixWith(
-        reassignment.get(), state.getSourceForNode(removedVariableTree.get().getType()) + " ");
+    return SuggestedFix.emptyFix();
   }
 
   private static String describeVariable(VarSymbol symbol) {
@@ -360,7 +372,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
     }
     ElementKind varKind = varSymbol.getKind();
     boolean encounteredSideEffects = false;
-    SuggestedFix.Builder fix = SuggestedFix.builder().setShortDescription("remove unused variable");
+    SuggestedFix.Builder keepSideEffectsFix =
+        SuggestedFix.builder().setShortDescription("remove unused variable");
     SuggestedFix.Builder removeSideEffectsFix =
         SuggestedFix.builder().setShortDescription("remove unused variable and any side effects");
     for (TreePath usagePath : usagePaths) {
@@ -378,10 +391,12 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
                 String.format(
                     "%s{ %s; }",
                     varSymbol.isStatic() ? "static " : "", state.getSourceForNode(initializer));
-            fix.merge(SuggestedFixes.replaceIncludingComments(usagePath, newContent, state));
+            keepSideEffectsFix.merge(
+                SuggestedFixes.replaceIncludingComments(usagePath, newContent, state));
             removeSideEffectsFix.replace(statement, "");
           } else {
-            fix.replace(statement, String.format("%s;", state.getSourceForNode(initializer)));
+            keepSideEffectsFix.replace(
+                statement, String.format("%s;", state.getSourceForNode(initializer)));
             removeSideEffectsFix.replace(statement, "");
           }
         } else if (isEnhancedForLoopVar(usagePath)) {
@@ -397,11 +412,12 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
                   state.getSourceForNode(variableTree.getType()));
           // The new content for the second fix should be identical to the content for the first
           // fix in this case because we can't just remove the enhanced for loop variable.
-          fix.replace(variableTree, newContent);
+          keepSideEffectsFix.replace(variableTree, newContent);
           removeSideEffectsFix.replace(variableTree, newContent);
         } else {
           String replacement = needsBlock(usagePath) ? "{}" : "";
-          fix.merge(SuggestedFixes.replaceIncludingComments(usagePath, replacement, state));
+          keepSideEffectsFix.merge(
+              SuggestedFixes.replaceIncludingComments(usagePath, replacement, state));
           removeSideEffectsFix.merge(
               SuggestedFixes.replaceIncludingComments(usagePath, replacement, state));
         }
@@ -418,14 +434,14 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
                     tree.getStartPosition(),
                     ((JCAssignOp) tree).getExpression().getStartPosition(),
                     "");
-            fix.merge(replacement);
+            keepSideEffectsFix.merge(replacement);
             removeSideEffectsFix.merge(replacement);
             continue;
           }
         } else if (tree instanceof AssignmentTree) {
           if (hasSideEffect(((AssignmentTree) tree).getExpression())) {
             encounteredSideEffects = true;
-            fix.replace(
+            keepSideEffectsFix.replace(
                 tree.getStartPosition(), ((JCAssign) tree).getExpression().getStartPosition(), "");
             removeSideEffectsFix.replace(statement, "");
             continue;
@@ -433,12 +449,12 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
         }
       }
       String replacement = needsBlock(usagePath) ? "{}" : "";
-      fix.replace(statement, replacement);
+      keepSideEffectsFix.replace(statement, replacement);
       removeSideEffectsFix.replace(statement, replacement);
     }
     return encounteredSideEffects
-        ? ImmutableList.of(removeSideEffectsFix.build(), fix.build())
-        : ImmutableList.of(fix.build());
+        ? ImmutableList.of(removeSideEffectsFix.build(), keepSideEffectsFix.build())
+        : ImmutableList.of(keepSideEffectsFix.build());
   }
 
   private static ImmutableList<SuggestedFix> buildUnusedParameterFixes(
@@ -473,7 +489,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
         }
         if (trees.size() == 1) {
           Tree tree = getOnlyElement(trees);
-          if (((JCTree) tree).getStartPosition() == -1 || state.getEndPosition(tree) == -1) {
+          if (getStartPosition(tree) == -1 || state.getEndPosition(tree) == -1) {
             // TODO(b/118437729): handle bogus source positions in enum declarations
             return;
           }
@@ -486,8 +502,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
           startPos = state.getEndPosition(trees.get(index - 1));
           endPos = state.getEndPosition(trees.get(index));
         } else {
-          startPos = ((JCTree) trees.get(index)).getStartPosition();
-          endPos = ((JCTree) trees.get(index + 1)).getStartPosition();
+          startPos = getStartPosition(trees.get(index));
+          endPos = getStartPosition(trees.get(index + 1));
         }
         if (index == methodSymbol.params().size() - 1 && methodSymbol.isVarArgs()) {
           endPos = state.getEndPosition(getLast(trees));
@@ -513,8 +529,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
    * Looks at the list of {@code annotations} and see if there is any annotation which exists {@code
    * exemptingAnnotations}.
    */
-  private static boolean exemptedByAnnotation(
-      List<? extends AnnotationTree> annotations, VisitorState state) {
+  private static boolean exemptedByAnnotation(List<? extends AnnotationTree> annotations) {
     for (AnnotationTree annotation : annotations) {
       Type annotationType = ASTHelpers.getType(annotation);
       if (annotationType == null) {
@@ -552,11 +567,13 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       if (exemptedByName(variableTree.getName())) {
         return null;
       }
-      if (isSuppressed(variableTree)) {
+      if (isSuppressed(variableTree, state)) {
         return null;
       }
       VarSymbol symbol = getSymbol(variableTree);
-      if (symbol == null) {
+      if (symbol.getKind() == ElementKind.FIELD
+          && symbol.getSimpleName().contentEquals("CREATOR")
+          && isSubtype(symbol.type, PARCELABLE_CREATOR.get(state), state)) {
         return null;
       }
       if (symbol.getKind() == ElementKind.FIELD
@@ -565,7 +582,8 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       }
       super.visitVariable(variableTree, null);
       // Return if the element is exempted by an annotation.
-      if (exemptedByAnnotation(variableTree.getModifiers().getAnnotations(), state)) {
+      if (exemptedByAnnotation(variableTree.getModifiers().getAnnotations())
+          || shouldKeep(variableTree)) {
         return null;
       }
       switch (symbol.getKind()) {
@@ -607,10 +625,14 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
           && ASTHelpers.hasDirectAnnotationWithSimpleName(variableTree, "Inject")) {
         return true;
       }
-      return variableTree.getModifiers().getFlags().contains(Modifier.PRIVATE)
-          && !SPECIAL_FIELDS.contains(symbol.getSimpleName().toString());
+      if ((symbol.flags() & RECORD_FLAG) == RECORD_FLAG) {
+        return false;
+      }
+      return canBeRemoved(symbol) && !SPECIAL_FIELDS.contains(symbol.getSimpleName().toString());
     }
 
+    private static final long RECORD_FLAG = 1L << 61;
+
     /** Returns whether {@code sym} can be removed without updating call sites in other files. */
     private boolean isParameterSubjectToAnalysis(Symbol sym) {
       checkArgument(sym.getKind() == ElementKind.PARAMETER);
@@ -622,7 +644,6 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
         }
       }
 
-
       return enclosingMethod.getModifiers().contains(Modifier.PRIVATE);
     }
 
@@ -637,7 +658,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
 
     @Override
     public Void visitClass(ClassTree tree, Void unused) {
-      if (isSuppressed(tree)) {
+      if (isSuppressed(tree, state)) {
         return null;
       }
       if (EXEMPTING_SUPER_TYPES.stream()
@@ -658,7 +679,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
       if (SERIALIZATION_METHODS.matches(tree, state)) {
         return scan(tree.getBody(), null);
       }
-      return isSuppressed(tree) ? null : super.visitMethod(tree, unused);
+      return isSuppressed(tree, state) ? null : super.visitMethod(tree, unused);
     }
   }
 
@@ -854,9 +875,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
     public Void visitMemberReference(MemberReferenceTree tree, Void unused) {
       super.visitMemberReference(tree, null);
       MethodSymbol symbol = getSymbol(tree);
-      if (symbol != null) {
-        symbol.getParameters().forEach(unusedElements::remove);
-      }
+      symbol.getParameters().forEach(unusedElements::remove);
       return null;
     }
 
@@ -933,7 +952,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
 
   @AutoValue
   abstract static class UnusedSpec {
-    /** {@link Symbol} of the unsued element. */
+    /** {@link Symbol} of the unused element. */
     abstract Symbol symbol();
 
     /** {@link VariableTree} or {@link AssignmentTree} for the original assignment site. */
@@ -963,4 +982,7 @@ public final class UnusedVariable extends BugChecker implements CompilationUnitT
           Optional.ofNullable(assignmentTree));
     }
   }
+
+  private static final Supplier<Type> PARCELABLE_CREATOR =
+      VisitorState.memoize(state -> state.getTypeFromString("android.os.Parcelable.Creator"));
 }
