@@ -17,11 +17,25 @@
 package com.google.errorprone.bugpatterns.threadsafety;
 
 import static com.google.common.collect.ImmutableSet.toImmutableSet;
+import static com.google.common.collect.Streams.stream;
 import static com.google.errorprone.BugPattern.SeverityLevel.ERROR;
 import static com.google.errorprone.matchers.Description.NO_MATCH;
+import static com.google.errorprone.util.ASTHelpers.getReceiver;
+import static com.google.errorprone.util.ASTHelpers.getReceiverType;
+import static com.google.errorprone.util.ASTHelpers.getSymbol;
+import static com.google.errorprone.util.ASTHelpers.getType;
+import static com.google.errorprone.util.ASTHelpers.hasAnnotation;
+import static com.google.errorprone.util.ASTHelpers.isLocal;
+import static com.google.errorprone.util.ASTHelpers.isSameType;
+import static com.google.errorprone.util.ASTHelpers.isSubtype;
+import static com.google.errorprone.util.ASTHelpers.targetType;
+import static java.lang.String.format;
+import static java.util.stream.Collectors.joining;
 
 import com.google.common.base.Joiner;
 import com.google.common.collect.ImmutableSet;
+import com.google.common.collect.LinkedHashMultimap;
+import com.google.common.collect.SetMultimap;
 import com.google.common.collect.Sets;
 import com.google.common.collect.Sets.SetView;
 import com.google.errorprone.BugPattern;
@@ -30,33 +44,46 @@ import com.google.errorprone.VisitorState;
 import com.google.errorprone.annotations.Immutable;
 import com.google.errorprone.bugpatterns.BugChecker;
 import com.google.errorprone.bugpatterns.BugChecker.ClassTreeMatcher;
+import com.google.errorprone.bugpatterns.BugChecker.LambdaExpressionTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.MemberReferenceTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.MethodInvocationTreeMatcher;
+import com.google.errorprone.bugpatterns.BugChecker.MethodTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.NewClassTreeMatcher;
-import com.google.errorprone.bugpatterns.BugChecker.TypeParameterTreeMatcher;
-import com.google.errorprone.bugpatterns.threadsafety.ImmutableAnalysis.ViolationReporter;
 import com.google.errorprone.bugpatterns.threadsafety.ThreadSafety.Violation;
 import com.google.errorprone.fixes.Fix;
 import com.google.errorprone.fixes.SuggestedFix;
 import com.google.errorprone.matchers.Description;
-import com.google.errorprone.matchers.Description.Builder;
 import com.google.errorprone.util.ASTHelpers;
 import com.sun.source.tree.ClassTree;
+import com.sun.source.tree.IdentifierTree;
+import com.sun.source.tree.LambdaExpressionTree;
 import com.sun.source.tree.MemberReferenceTree;
+import com.sun.source.tree.MemberSelectTree;
 import com.sun.source.tree.MethodInvocationTree;
+import com.sun.source.tree.MethodTree;
 import com.sun.source.tree.NewClassTree;
 import com.sun.source.tree.Tree;
 import com.sun.source.tree.TypeParameterTree;
+import com.sun.source.tree.VariableTree;
+import com.sun.source.util.TreePath;
+import com.sun.source.util.TreePathScanner;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.code.Symbol.ClassSymbol;
+import com.sun.tools.javac.code.Symbol.MethodSymbol;
+import com.sun.tools.javac.code.Symbol.TypeSymbol;
 import com.sun.tools.javac.code.Symbol.TypeVariableSymbol;
+import com.sun.tools.javac.code.Symbol.VarSymbol;
 import com.sun.tools.javac.code.Type;
+import com.sun.tools.javac.code.Type.ClassType;
 import com.sun.tools.javac.tree.JCTree.JCMemberReference;
 import com.sun.tools.javac.tree.JCTree.JCNewClass;
+import java.util.Collection;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Map;
-import java.util.Map.Entry;
 import java.util.Optional;
+import java.util.Set;
+import javax.lang.model.element.ElementKind;
 
 /** A {@link BugChecker}; see the associated {@link BugPattern} annotation for details. */
 @BugPattern(
@@ -66,18 +93,15 @@ import java.util.Optional;
     documentSuppression = false)
 public class ImmutableChecker extends BugChecker
     implements ClassTreeMatcher,
+        LambdaExpressionTreeMatcher,
         NewClassTreeMatcher,
-        TypeParameterTreeMatcher,
         MethodInvocationTreeMatcher,
+        MethodTreeMatcher,
         MemberReferenceTreeMatcher {
 
   private final WellKnownMutability wellKnownMutability;
   private final ImmutableSet<String> immutableAnnotations;
-
-  @Deprecated // Used reflectively, but you should pass in ErrorProneFlags to get custom mutability
-  public ImmutableChecker() {
-    this(ErrorProneFlags.empty());
-  }
+  private final boolean handleAnonymousClasses;
 
   ImmutableChecker(ImmutableSet<String> immutableAnnotations) {
     this(ErrorProneFlags.empty(), immutableAnnotations);
@@ -90,36 +114,107 @@ public class ImmutableChecker extends BugChecker
   private ImmutableChecker(ErrorProneFlags flags, ImmutableSet<String> immutableAnnotations) {
     this.wellKnownMutability = WellKnownMutability.fromFlags(flags);
     this.immutableAnnotations = immutableAnnotations;
+    this.handleAnonymousClasses =
+        flags.getBoolean("ImmutableChecker:HandleAnonymousClasses").orElse(true);
+  }
+
+  @Override
+  public Description matchLambdaExpression(LambdaExpressionTree tree, VisitorState state) {
+    TypeSymbol lambdaType = getType(tree).tsym;
+    ImmutableAnalysis analysis = createImmutableAnalysis(state);
+    Violation info =
+        analysis.checkInstantiation(
+            lambdaType.getTypeParameters(), getType(tree).getTypeArguments());
+
+    if (info.isPresent()) {
+      state.reportMatch(buildDescription(tree).setMessage(info.message()).build());
+    }
+    if (!hasImmutableAnnotation(lambdaType, state)) {
+      return NO_MATCH;
+    }
+    checkClosedTypes(tree, state, lambdaType, analysis);
+
+    return NO_MATCH;
+  }
+
+  private boolean hasImmutableAnnotation(TypeSymbol tsym, VisitorState state) {
+    return immutableAnnotations.stream()
+        .anyMatch(annotation -> hasAnnotation(tsym, annotation, state));
   }
 
-  // check instantiations of `@ImmutableTypeParameter`s in method references
   @Override
   public Description matchMemberReference(MemberReferenceTree tree, VisitorState state) {
-    return checkInvocation(
-        tree, ((JCMemberReference) tree).referentType, state, ASTHelpers.getSymbol(tree));
+    // check instantiations of `@ImmutableTypeParameter`s in method references
+    checkInvocation(tree, getSymbol(tree), ((JCMemberReference) tree).referentType, state);
+    ImmutableAnalysis analysis = createImmutableAnalysis(state);
+    TypeSymbol memberReferenceType = targetType(state).type().tsym;
+    Violation info =
+        analysis.checkInstantiation(
+            memberReferenceType.getTypeParameters(), getType(tree).getTypeArguments());
+
+    if (info.isPresent()) {
+      state.reportMatch(buildDescription(tree).setMessage(info.message()).build());
+    }
+    if (!hasImmutableAnnotation(memberReferenceType, state)) {
+      return NO_MATCH;
+    }
+    if (getSymbol(getReceiver(tree)) instanceof ClassSymbol) {
+      return NO_MATCH;
+    }
+    var receiverType = getReceiverType(tree);
+    ImmutableSet<String> typarams =
+        immutableTypeParametersInScope(getSymbol(tree), state, analysis);
+    var violation =
+        analysis.isThreadSafeType(/* allowContainerTypeParameters= */ true, typarams, receiverType);
+    if (violation.isPresent()) {
+      return buildDescription(tree)
+          .setMessage(
+              "This method reference implements @Immutable interface "
+                  + memberReferenceType.getSimpleName()
+                  + ", but "
+                  + violation.message())
+          .build();
+    }
+    return NO_MATCH;
   }
 
   // check instantiations of `@ImmutableTypeParameter`s in method invocations
   @Override
   public Description matchMethodInvocation(MethodInvocationTree tree, VisitorState state) {
-    return checkInvocation(
-        tree, ASTHelpers.getType(tree.getMethodSelect()), state, ASTHelpers.getSymbol(tree));
+    checkInvocation(tree, getSymbol(tree), ASTHelpers.getType(tree.getMethodSelect()), state);
+    return NO_MATCH;
   }
 
   @Override
   public Description matchNewClass(NewClassTree tree, VisitorState state) {
     // check instantiations of `@ImmutableTypeParameter`s in generic constructor invocations
-    checkInvocation(
-        tree, ((JCNewClass) tree).constructorType, state, ((JCNewClass) tree).constructor);
+    checkInvocation(tree, getSymbol(tree), ((JCNewClass) tree).constructorType, state);
     // check instantiations of `@ImmutableTypeParameter`s in class constructor invocations
-    ImmutableAnalysis analysis = createImmutableAnalysis(state);
-    Violation info =
-        analysis.checkInstantiation(
-            ASTHelpers.getSymbol(tree.getIdentifier()).getTypeParameters(),
-            ASTHelpers.getType(tree).getTypeArguments());
-    if (info.isPresent()) {
-      state.reportMatch(buildDescription(tree).setMessage(info.message()).build());
+    checkInstantiation(
+        tree,
+        state,
+        getSymbol(tree.getIdentifier()).getTypeParameters(),
+        ASTHelpers.getType(tree).getTypeArguments());
+
+    ClassTree classBody = tree.getClassBody();
+
+    // Only anonymous classes have a body next to the new operator.
+    if (classBody != null) {
+      // check instantiations of `@ImmutableTypeParameter`s in anonymous class constructor
+      // invocations
+      checkClassTreeInstantiation(classBody, state, createImmutableAnalysis(state));
     }
+
+    return NO_MATCH;
+  }
+
+  @Override
+  public Description matchMethod(MethodTree tree, VisitorState state) {
+    checkInstantiation(
+        tree,
+        state,
+        getSymbol(tree).getTypeParameters(),
+        ASTHelpers.getType(tree).getTypeArguments());
     return NO_MATCH;
   }
 
@@ -127,44 +222,41 @@ public class ImmutableChecker extends BugChecker
     return new ImmutableAnalysis(this, state, wellKnownMutability, immutableAnnotations);
   }
 
-  private Description checkInvocation(
-      Tree tree, Type methodType, VisitorState state, Symbol symbol) {
+  private void checkInvocation(
+      Tree tree, MethodSymbol symbol, Type methodType, VisitorState state) {
     ImmutableAnalysis analysis = createImmutableAnalysis(state);
     Violation info = analysis.checkInvocation(methodType, symbol);
     if (info.isPresent()) {
       state.reportMatch(buildDescription(tree).setMessage(info.message()).build());
     }
-    return NO_MATCH;
   }
 
-  @Override
-  public Description matchTypeParameter(TypeParameterTree tree, VisitorState state) {
-    Symbol sym = ASTHelpers.getSymbol(tree);
-    if (sym == null) {
-      return NO_MATCH;
+  private void checkInstantiation(
+      Tree tree,
+      VisitorState state,
+      ImmutableAnalysis analysis,
+      Collection<TypeVariableSymbol> typeParameters,
+      Collection<Type> typeArguments) {
+    Violation info = analysis.checkInstantiation(typeParameters, typeArguments);
+
+    if (info.isPresent()) {
+      state.reportMatch(buildDescription(tree).setMessage(info.message()).build());
     }
-    ImmutableAnalysis analysis = createImmutableAnalysis(state);
-    if (!analysis.hasThreadSafeTypeParameterAnnotation((TypeVariableSymbol) sym)) {
-      return NO_MATCH;
-    }
-    switch (sym.owner.getKind()) {
-      case METHOD:
-      case CONSTRUCTOR:
-        return NO_MATCH;
-      default: // fall out
-    }
-    AnnotationInfo info = analysis.getImmutableAnnotation(sym.owner, state);
-    if (info == null) {
-      return buildDescription(tree)
-          .setMessage("@Immutable is only supported on immutable classes")
-          .build();
-    }
-    return NO_MATCH;
+  }
+
+  private void checkInstantiation(
+      Tree tree,
+      VisitorState state,
+      Collection<TypeVariableSymbol> typeParameters,
+      Collection<Type> typeArguments) {
+    checkInstantiation(tree, state, createImmutableAnalysis(state), typeParameters, typeArguments);
   }
 
   @Override
   public Description matchClass(ClassTree tree, VisitorState state) {
     ImmutableAnalysis analysis = createImmutableAnalysis(state);
+    checkClassTreeInstantiation(tree, state, analysis);
+
     if (tree.getSimpleName().length() == 0) {
       // anonymous classes have empty names
       // TODO(cushon): once Java 8 happens, require @Immutable on anonymous classes
@@ -187,14 +279,13 @@ public class ImmutableChecker extends BugChecker
     // Check that the types in containerOf actually exist
     Map<String, TypeVariableSymbol> typarams = new HashMap<>();
     for (TypeParameterTree typaram : tree.getTypeParameters()) {
-      typarams.put(
-          typaram.getName().toString(), (TypeVariableSymbol) ASTHelpers.getSymbol(typaram));
+      typarams.put(typaram.getName().toString(), (TypeVariableSymbol) getSymbol(typaram));
     }
     SetView<String> difference = Sets.difference(annotation.containerOf(), typarams.keySet());
     if (!difference.isEmpty()) {
       return buildDescription(tree)
           .setMessage(
-              String.format(
+              format(
                   "could not find type(s) referenced by containerOf: %s",
                   Joiner.on("', '").join(difference)))
           .build();
@@ -205,12 +296,12 @@ public class ImmutableChecker extends BugChecker
                 e ->
                     annotation.containerOf().contains(e.getKey())
                         && analysis.hasThreadSafeTypeParameterAnnotation(e.getValue()))
-            .map(Entry::getKey)
+            .map(Map.Entry::getKey)
             .collect(toImmutableSet());
     if (!immutableAndContainer.isEmpty()) {
       return buildDescription(tree)
           .setMessage(
-              String.format(
+              format(
                   "using both @ImmutableTypeParameter and containerOf is redundant: %s",
                   Joiner.on("', '").join(immutableAndContainer)))
           .build();
@@ -221,16 +312,21 @@ public class ImmutableChecker extends BugChecker
     // Check that the fields (including inherited fields) are immutable, and
     // validate the type hierarchy superclass.
 
-    ClassSymbol sym = ASTHelpers.getSymbol(tree);
+    ClassSymbol sym = getSymbol(tree);
 
     Violation info =
         analysis.checkForImmutability(
             Optional.of(tree),
-            immutableTypeParametersInScope(ASTHelpers.getSymbol(tree), state, analysis),
+            immutableTypeParametersInScope(getSymbol(tree), state, analysis),
             ASTHelpers.getType(tree),
             (Tree matched, Violation violation) ->
                 describeClass(matched, sym, annotation, violation));
 
+    Type superType = immutableSupertype(sym, state);
+    if (handleAnonymousClasses && superType != null && isLocal(sym)) {
+      checkClosedTypes(tree, state, superType.tsym, analysis);
+    }
+
     if (!info.isPresent()) {
       return NO_MATCH;
     }
@@ -238,6 +334,28 @@ public class ImmutableChecker extends BugChecker
     return describeClass(tree, sym, annotation, info).build();
   }
 
+  private void checkClassTreeInstantiation(
+      ClassTree tree, VisitorState state, ImmutableAnalysis analysis) {
+    for (Tree implementTree : tree.getImplementsClause()) {
+      checkInstantiation(
+          tree,
+          state,
+          analysis,
+          getSymbol(implementTree).getTypeParameters(),
+          ASTHelpers.getType(implementTree).getTypeArguments());
+    }
+
+    Tree extendsClause = tree.getExtendsClause();
+    if (extendsClause != null) {
+      checkInstantiation(
+          tree,
+          state,
+          analysis,
+          getSymbol(extendsClause).getTypeParameters(),
+          ASTHelpers.getType(extendsClause).getTypeArguments());
+    }
+  }
+
   private Description.Builder describeClass(
       Tree tree, ClassSymbol sym, AnnotationInfo annotation, Violation info) {
     String message;
@@ -245,7 +363,7 @@ public class ImmutableChecker extends BugChecker
       message = "type annotated with @Immutable could not be proven immutable: " + info.message();
     } else {
       message =
-          String.format(
+          format(
               "Class extends @Immutable type %s, but is not immutable: %s",
               annotation.typeName(), info.message());
     }
@@ -257,14 +375,15 @@ public class ImmutableChecker extends BugChecker
   /** Check anonymous implementations of {@code @Immutable} types. */
   private Description handleAnonymousClass(
       ClassTree tree, VisitorState state, ImmutableAnalysis analysis) {
-    ClassSymbol sym = ASTHelpers.getSymbol(tree);
-    if (sym == null) {
-      return NO_MATCH;
-    }
+    ClassSymbol sym = getSymbol(tree);
     Type superType = immutableSupertype(sym, state);
     if (superType == null) {
       return NO_MATCH;
     }
+
+    if (handleAnonymousClasses) {
+      checkClosedTypes(tree, state, superType.tsym, analysis);
+    }
     // We don't need to check that the superclass has an immutable instantiation.
     // The anonymous instance can only be referred to using a superclass type, so
     // the type arguments will be validated at any type use site where we care about
@@ -281,21 +400,145 @@ public class ImmutableChecker extends BugChecker
             Optional.of(tree),
             typarams,
             ASTHelpers.getType(tree),
-            new ViolationReporter() {
-              @Override
-              public Builder describe(Tree tree, Violation info) {
-                return describeAnonymous(tree, superType, info);
-              }
-            });
+            (t, i) -> describeAnonymous(t, superType, i));
     if (!info.isPresent()) {
       return NO_MATCH;
     }
     return describeAnonymous(tree, superType, info).build();
   }
 
+  private void checkClosedTypes(
+      Tree lambdaOrAnonymousClass,
+      VisitorState state,
+      TypeSymbol lambdaType,
+      ImmutableAnalysis analysis) {
+    Set<VarSymbol> variablesClosed = new HashSet<>();
+    SetMultimap<ClassSymbol, MethodSymbol> typesClosed = LinkedHashMultimap.create();
+    Set<VarSymbol> variablesOwnedByLambda = new HashSet<>();
+
+    new TreePathScanner<Void, Void>() {
+      @Override
+      public Void visitVariable(VariableTree tree, Void unused) {
+        var symbol = getSymbol(tree);
+        variablesOwnedByLambda.add(symbol);
+        return super.visitVariable(tree, null);
+      }
+
+      @Override
+      public Void visitMethodInvocation(MethodInvocationTree tree, Void unused) {
+        if (getReceiver(tree) == null) {
+          var symbol = getSymbol(tree);
+          if (!symbol.isStatic() && !symbol.isConstructor()) {
+            effectiveTypeOfThis(symbol, getCurrentPath(), state)
+                .filter(t -> !isSameType(t.type, getType(lambdaOrAnonymousClass), state))
+                .ifPresent(t -> typesClosed.put(t, symbol));
+          }
+        }
+        return super.visitMethodInvocation(tree, null);
+      }
+
+      @Override
+      public Void visitMemberSelect(MemberSelectTree tree, Void unused) {
+        // Note: member selects are not intrinsically problematic; the issue is what might be on the
+        // LHS of them, which is going to be handled by another visit* method.
+
+        // If we're only seeing a field access, don't complain about the fact we closed around
+        // `this`. This is special-case as it would otherwise be vexing to complain about accessing
+        // a field of type ImmutableList.
+        if (tree.getExpression() instanceof IdentifierTree
+            && getSymbol(tree) instanceof VarSymbol
+            && ((IdentifierTree) tree.getExpression()).getName().contentEquals("this")) {
+          handleIdentifier(getSymbol(tree));
+          return null;
+        }
+        return super.visitMemberSelect(tree, null);
+      }
+
+      @Override
+      public Void visitIdentifier(IdentifierTree tree, Void unused) {
+        handleIdentifier(getSymbol(tree));
+        return super.visitIdentifier(tree, null);
+      }
+
+      private void handleIdentifier(Symbol symbol) {
+        if (symbol instanceof VarSymbol
+            && !variablesOwnedByLambda.contains(symbol)
+            && !symbol.isStatic()) {
+          variablesClosed.add((VarSymbol) symbol);
+        }
+      }
+    }.scan(state.getPath(), null);
+
+    ImmutableSet<String> typarams =
+        immutableTypeParametersInScope(getSymbol(lambdaOrAnonymousClass), state, analysis);
+    variablesClosed.stream()
+        .map(
+            closedVariable ->
+                checkClosedVariable(closedVariable, lambdaOrAnonymousClass, typarams, analysis))
+        .filter(Violation::isPresent)
+        .forEachOrdered(
+            v -> {
+              String message =
+                  formAnonymousReason(lambdaOrAnonymousClass, lambdaType) + ", but " + v.message();
+              state.reportMatch(
+                  buildDescription(lambdaOrAnonymousClass).setMessage(message).build());
+            });
+    for (var entry : typesClosed.asMap().entrySet()) {
+      var classSymbol = entry.getKey();
+      var methods = entry.getValue();
+      if (!hasImmutableAnnotation(classSymbol.type.tsym, state)) {
+        String message =
+            format(
+                "%s, but accesses instance method(s) '%s' on '%s' which is not @Immutable.",
+                formAnonymousReason(lambdaOrAnonymousClass, lambdaType),
+                methods.stream().map(Symbol::getSimpleName).collect(joining(", ")),
+                classSymbol.getSimpleName());
+        state.reportMatch(buildDescription(lambdaOrAnonymousClass).setMessage(message).build());
+      }
+    }
+  }
+
+  /**
+   * Gets the effective type of `this`, had the bare invocation of {@code symbol} been qualified
+   * with it.
+   */
+  private static Optional<ClassSymbol> effectiveTypeOfThis(
+      MethodSymbol symbol, TreePath currentPath, VisitorState state) {
+    return stream(currentPath.iterator())
+        .filter(ClassTree.class::isInstance)
+        .map(t -> ASTHelpers.getSymbol((ClassTree) t))
+        .filter(c -> isSubtype(c.type, symbol.owner.type, state))
+        .findFirst();
+  }
+
+  private Violation checkClosedVariable(
+      VarSymbol closedVariable,
+      Tree tree,
+      ImmutableSet<String> typarams,
+      ImmutableAnalysis analysis) {
+    if (!closedVariable.getKind().equals(ElementKind.FIELD)) {
+      return analysis.isThreadSafeType(false, typarams, closedVariable.type);
+    }
+    return analysis.isFieldImmutable(
+        Optional.empty(),
+        typarams,
+        (ClassSymbol) closedVariable.owner,
+        (ClassType) closedVariable.owner.type,
+        closedVariable,
+        (t, v) -> buildDescription(tree));
+  }
+
+  private static String formAnonymousReason(Tree tree, TypeSymbol typeSymbol) {
+    return "This "
+        + (tree instanceof LambdaExpressionTree ? "lambda" : "anonymous class")
+        + " implements @Immutable interface '"
+        + typeSymbol.getSimpleName()
+        + "'";
+  }
+
   private Description.Builder describeAnonymous(Tree tree, Type superType, Violation info) {
     String message =
-        String.format(
+        format(
             "Class extends @Immutable type %s, but is not immutable: %s",
             superType, info.message());
     return buildDescription(tree).setMessage(message);
@@ -305,17 +548,13 @@ public class ImmutableChecker extends BugChecker
 
   /** Check for classes without {@code @Immutable} that have immutable supertypes. */
   private Description checkSubtype(ClassTree tree, VisitorState state) {
-    ClassSymbol sym = ASTHelpers.getSymbol(tree);
-    if (sym == null) {
-      return NO_MATCH;
-    }
+    ClassSymbol sym = getSymbol(tree);
     Type superType = immutableSupertype(sym, state);
     if (superType == null) {
       return NO_MATCH;
     }
     String message =
-        String.format(
-            "Class extends @Immutable type %s, but is not annotated as immutable", superType);
+        format("Class extends @Immutable type %s, but is not annotated as immutable", superType);
     Fix fix =
         SuggestedFix.builder()
             .prefixWith(tree, "@Immutable ")
@@ -335,8 +574,7 @@ public class ImmutableChecker extends BugChecker
       }
       // Don't use getImmutableAnnotation here: subtypes of trusted types are
       // also trusted, only check for explicitly annotated supertypes.
-      if (immutableAnnotations.stream()
-          .anyMatch(annotation -> ASTHelpers.hasAnnotation(superType.tsym, annotation, state))) {
+      if (hasImmutableAnnotation(superType.tsym, state)) {
         return superType;
       }
       // We currently trust that @interface annotations are immutable, but don't enforce that
