@@ -20,6 +20,7 @@ import static com.google.errorprone.matchers.Matchers.anyOf;
 import static com.google.errorprone.matchers.Matchers.constructor;
 import static com.google.errorprone.matchers.method.MethodMatchers.instanceMethod;
 import static com.google.errorprone.matchers.method.MethodMatchers.staticMethod;
+import static com.google.errorprone.util.ASTHelpers.getStartPosition;
 import static com.google.errorprone.util.ASTHelpers.getSymbol;
 import static com.google.errorprone.util.ASTHelpers.isSameType;
 import static com.google.errorprone.util.ASTHelpers.isSubtype;
@@ -32,7 +33,6 @@ import static java.util.concurrent.TimeUnit.MINUTES;
 import static java.util.concurrent.TimeUnit.NANOSECONDS;
 import static java.util.concurrent.TimeUnit.SECONDS;
 
-import com.google.common.base.Predicate;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Iterables;
 import com.google.errorprone.BugPattern;
@@ -44,6 +44,7 @@ import com.google.errorprone.fixes.SuggestedFix;
 import com.google.errorprone.fixes.SuggestedFixes;
 import com.google.errorprone.matchers.Description;
 import com.google.errorprone.matchers.Matcher;
+import com.google.errorprone.suppliers.Supplier;
 import com.google.errorprone.util.ASTHelpers;
 import com.sun.source.tree.ExpressionTree;
 import com.sun.source.tree.MethodInvocationTree;
@@ -56,18 +57,16 @@ import com.sun.tools.javac.code.Symbol.TypeSymbol;
 import com.sun.tools.javac.code.Symbol.VarSymbol;
 import com.sun.tools.javac.code.Type;
 import com.sun.tools.javac.code.Types;
-import com.sun.tools.javac.tree.JCTree;
-import com.sun.tools.javac.util.Filter;
 import com.sun.tools.javac.util.Name;
 import java.util.List;
-import java.util.Map.Entry;
+import java.util.Map;
 import java.util.Optional;
 import java.util.concurrent.TimeUnit;
+import java.util.function.Predicate;
 import javax.annotation.Nullable;
 
 /** This check suggests the use of {@code java.time}-based APIs, when available. */
 @BugPattern(
-    name = "PreferJavaTimeOverload",
     altNames = {"PreferDurationOverload"},
     summary =
         "Prefer using java.time-based APIs when available. Note that this checker does"
@@ -89,7 +88,7 @@ public final class PreferJavaTimeOverload extends BugChecker
               .put(staticMethod().onClass(JODA_DURATION).named("standardMinutes"), MINUTES)
               .put(staticMethod().onClass(JODA_DURATION).named("standardHours"), HOURS)
               .put(staticMethod().onClass(JODA_DURATION).named("standardDays"), DAYS)
-              .build();
+              .buildOrThrow();
 
   private static final ImmutableMap<TimeUnit, String> TIMEUNIT_TO_DURATION_FACTORY =
       new ImmutableMap.Builder<TimeUnit, String>()
@@ -100,7 +99,7 @@ public final class PreferJavaTimeOverload extends BugChecker
           .put(MINUTES, "%s.ofMinutes(%s)")
           .put(HOURS, "%s.ofHours(%s)")
           .put(DAYS, "%s.ofDays(%s)")
-          .build();
+          .buildOrThrow();
 
   private static final String JAVA_INSTANT = "java.time.Instant";
   private static final String JODA_INSTANT = "org.joda.time.Instant";
@@ -108,6 +107,9 @@ public final class PreferJavaTimeOverload extends BugChecker
   private static final Matcher<ExpressionTree> JODA_INSTANT_CONSTRUCTOR_MATCHER =
       constructor().forClass(JODA_INSTANT).withParameters("long");
 
+  private static final String TIME_SOURCE = "com.google.common.time.TimeSource";
+  private static final String JODA_CLOCK = "com.google.common.time.Clock";
+
   private static final String JAVA_TIME_CONVERSIONS =
       "com.google.thirdparty.jodatime.JavaTimeConversions";
 
@@ -118,11 +120,13 @@ public final class PreferJavaTimeOverload extends BugChecker
 
   private static final Matcher<ExpressionTree> IGNORED_APIS =
       anyOf(
-          staticMethod().onClass("org.jooq.impl.DSL").named("inline"),
+          staticMethod().onClass("org.jooq.impl.DSL").withAnyName(),
           // any static method under org.assertj.*
           staticMethod()
               .onClass((type, state) -> type.toString().startsWith("org.assertj."))
-              .withAnyName());
+              .withAnyName(),
+          // any instance method on Reactor's Flux API
+          instanceMethod().onDescendantOf("reactor.core.publisher.Flux").withAnyName());
 
   private static final Matcher<ExpressionTree> JAVA_DURATION_DECOMPOSITION_MATCHER =
       instanceMethod()
@@ -181,7 +185,7 @@ public final class PreferJavaTimeOverload extends BugChecker
               if (JAVA_DURATION_DECOMPOSITION_MATCHER.matches(maybeDurationDecomposition, state)) {
                 if (isSameType(
                     ASTHelpers.getReceiverType(maybeDurationDecomposition),
-                    state.getTypeFromString(JAVA_DURATION),
+                    JAVA_TIME_DURATION.get(state),
                     state)) {
                   replacement =
                       state.getSourceForNode(ASTHelpers.getReceiver(maybeDurationDecomposition));
@@ -204,7 +208,7 @@ public final class PreferJavaTimeOverload extends BugChecker
             }
 
             fix.replace(
-                ((JCTree) arguments.get(0)).getStartPosition(),
+                getStartPosition(arguments.get(0)),
                 state.getEndPosition(arguments.get(1)),
                 replacement);
             return describeMatch(tree, fix.build());
@@ -223,7 +227,7 @@ public final class PreferJavaTimeOverload extends BugChecker
         // TODO(kak): Add support for org.joda.time.Duration.ZERO -> java.time.Duration.ZERO
 
         // If the Joda Duration is being constructed inline, then unwrap it.
-        for (Entry<Matcher<ExpressionTree>, TimeUnit> entry :
+        for (Map.Entry<Matcher<ExpressionTree>, TimeUnit> entry :
             JODA_DURATION_FACTORY_MATCHERS.entrySet()) {
           if (entry.getKey().matches(arg0, state)) {
             String value = null;
@@ -339,7 +343,7 @@ public final class PreferJavaTimeOverload extends BugChecker
 
   private static boolean isLongTimeUnitMethodCall(MethodInvocationTree tree, VisitorState state) {
     Type longType = state.getSymtab().longType;
-    Type timeUnitType = state.getTypeFromString("java.util.concurrent.TimeUnit");
+    Type timeUnitType = JAVA_UTIL_CONCURRENT_TIMEUNIT.get(state);
     List<VarSymbol> params = getSymbol(tree).getParameters();
     if (params.size() == 2) {
       return isSameType(params.get(0).asType(), longType, state)
@@ -351,16 +355,18 @@ public final class PreferJavaTimeOverload extends BugChecker
   private static boolean hasJavaTimeOverload(
       MethodInvocationTree tree, VisitorState state, String typeName) {
     MethodSymbol calledMethod = getSymbol(tree);
-    if (calledMethod == null) {
-      return false;
-    }
+    return hasJavaTimeOverload(state, typeName, calledMethod, calledMethod.name);
+  }
+
+  private static boolean hasJavaTimeOverload(
+      VisitorState state, String typeName, MethodSymbol calledMethod, Name methodName) {
 
     MethodTree t = state.findEnclosing(MethodTree.class);
     @Nullable MethodSymbol enclosingMethod = t == null ? null : getSymbol(t);
 
     Type type = state.getTypeFromString(typeName);
     return hasMatchingMethods(
-        calledMethod.name,
+        methodName,
         input ->
             !input.equals(calledMethod)
                 // Make sure we're not currently *inside* that overload, to avoid
@@ -377,11 +383,18 @@ public final class PreferJavaTimeOverload extends BugChecker
         state.getTypes());
   }
 
+  private static boolean hasTimeSourceMethod(MethodInvocationTree tree, VisitorState state) {
+    MethodSymbol calledMethod = getSymbol(tree);
+    String timeSourceBasedName = calledMethod.name.toString().replace("Clock", "TimeSource");
+    return hasJavaTimeOverload(
+        state, TIME_SOURCE, calledMethod, state.getName(timeSourceBasedName));
+  }
+
   // Adapted from ASTHelpers.findMatchingMethods(); but this short-circuits
   private static boolean hasMatchingMethods(
-      Name name, final Predicate<MethodSymbol> predicate, Type startClass, Types types) {
-    Filter<Symbol> matchesMethodPredicate =
-        sym -> sym instanceof MethodSymbol && predicate.apply((MethodSymbol) sym);
+      Name name, Predicate<MethodSymbol> predicate, Type startClass, Types types) {
+    Predicate<Symbol> matchesMethodPredicate =
+        sym -> sym instanceof MethodSymbol && predicate.test((MethodSymbol) sym);
 
     // Iterate over all classes and interfaces that startClass inherits from.
     for (Type superClass : types.closure(startClass)) {
@@ -390,11 +403,18 @@ public final class PreferJavaTimeOverload extends BugChecker
       Scope superClassSymbols = superClassSymbol.members();
       if (superClassSymbols != null) { // Can be null if superClass is a type variable
         if (!Iterables.isEmpty(
-            superClassSymbols.getSymbolsByName(name, matchesMethodPredicate, NON_RECURSIVE))) {
+            ASTHelpers.scope(superClassSymbols)
+                .getSymbolsByName(name, matchesMethodPredicate, NON_RECURSIVE))) {
           return true;
         }
       }
     }
     return false;
   }
+
+  private static final Supplier<Type> JAVA_TIME_DURATION =
+      VisitorState.memoize(state -> state.getTypeFromString(JAVA_DURATION));
+
+  private static final Supplier<Type> JAVA_UTIL_CONCURRENT_TIMEUNIT =
+      VisitorState.memoize(state -> state.getTypeFromString("java.util.concurrent.TimeUnit"));
 }
