@@ -19,7 +19,9 @@ package com.google.errorprone;
 import static com.google.common.truth.Truth.assertThat;
 import static com.google.errorprone.BugPattern.SeverityLevel.ERROR;
 import static com.google.errorprone.DiagnosticTestHelper.diagnosticMessage;
+import static com.google.errorprone.FileObjects.forResources;
 import static java.nio.charset.StandardCharsets.UTF_8;
+import static java.util.Locale.ENGLISH;
 import static org.hamcrest.Matchers.containsString;
 import static org.hamcrest.Matchers.hasItem;
 import static org.junit.Assert.fail;
@@ -42,13 +44,13 @@ import com.google.errorprone.scanner.ScannerSupplier;
 import com.google.errorprone.util.ASTHelpers;
 import com.sun.source.tree.ClassTree;
 import com.sun.source.tree.MethodTree;
+import com.sun.tools.javac.file.JavacFileManager;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
-import java.io.UncheckedIOException;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
@@ -65,7 +67,9 @@ import org.junit.rules.TemporaryFolder;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
 
-/** @author cushon@google.com (Liam Miller-Cushon) */
+/**
+ * @author cushon@google.com (Liam Miller-Cushon)
+ */
 @RunWith(JUnit4.class)
 public class ErrorProneJavaCompilerTest {
 
@@ -284,7 +288,7 @@ public class ErrorProneJavaCompilerTest {
     DiagnosticTestHelper diagnosticHelper = new DiagnosticTestHelper();
     ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
     PrintWriter printWriter = new PrintWriter(new OutputStreamWriter(outputStream, UTF_8), true);
-    ErrorProneInMemoryFileManager fileManager = new ErrorProneInMemoryFileManager();
+    JavacFileManager fileManager = FileManagers.testFileManager();
     JavaCompiler errorProneJavaCompiler = new ErrorProneJavaCompiler();
     List<String> args =
         Lists.newArrayList(
@@ -292,11 +296,10 @@ public class ErrorProneJavaCompilerTest {
             tempDir.getRoot().getAbsolutePath(),
             "-proc:none",
             "-Xep:ChainingConstructorIgnoresParameter:WARN");
-    List<JavaFileObject> sources =
-        fileManager.forResources(
+    ImmutableList<JavaFileObject> sources =
+        forResources(
             ChainingConstructorIgnoresParameter.class,
             "testdata/ChainingConstructorIgnoresParameterPositiveCases.java");
-    fileManager.close();
 
     JavaCompiler.CompilationTask task =
         errorProneJavaCompiler.getTask(
@@ -309,12 +312,10 @@ public class ErrorProneJavaCompilerTest {
 
     // reset state between compilations
     diagnosticHelper.clearDiagnostics();
-    fileManager = new ErrorProneInMemoryFileManager();
     sources =
-        fileManager.forResources(
+        forResources(
             ChainingConstructorIgnoresParameter.class,
             "testdata/ChainingConstructorIgnoresParameterPositiveCases.java");
-    fileManager.close();
     args.remove("-Xep:ChainingConstructorIgnoresParameter:WARN");
 
     task =
@@ -330,14 +331,11 @@ public class ErrorProneJavaCompilerTest {
     DiagnosticTestHelper diagnosticHelper = new DiagnosticTestHelper();
     ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
     PrintWriter printWriter = new PrintWriter(new OutputStreamWriter(outputStream, UTF_8), true);
-    ErrorProneInMemoryFileManager fileManager = new ErrorProneInMemoryFileManager();
     JavaCompiler errorProneJavaCompiler = new ErrorProneJavaCompiler();
     List<String> args =
         Lists.newArrayList("-d", tempDir.getRoot().getAbsolutePath(), "-proc:none", "-Xep:EmptyIf");
-    List<JavaFileObject> sources =
-        fileManager.forResources(
-            BadShiftAmount.class, "testdata/EmptyIfStatementPositiveCases.java");
-    fileManager.close();
+    ImmutableList<JavaFileObject> sources =
+        forResources(BadShiftAmount.class, "testdata/EmptyIfStatementPositiveCases.java");
 
     JavaCompiler.CompilationTask task =
         errorProneJavaCompiler.getTask(
@@ -353,14 +351,12 @@ public class ErrorProneJavaCompilerTest {
     task =
         errorProneJavaCompiler.getTask(
             printWriter, null, diagnosticHelper.collector, args, null, sources);
-    fileManager.close();
     succeeded = task.call();
     assertThat(succeeded).isTrue();
     assertThat(diagnosticHelper.getDiagnostics()).isEmpty();
   }
 
   @BugPattern(
-      name = "DeleteMethod",
       summary =
           "You appear to be using methods; prefer to implement all program logic inside the main"
               + " function by flipping bits in a single long[].",
@@ -384,11 +380,11 @@ public class ErrorProneJavaCompilerTest {
             Collections.<String>emptyList(),
             ImmutableList.<Class<? extends BugChecker>>of(DeleteMethod.class));
     assertThat(result.succeeded).isFalse();
+    assertThat(result.output).isEmpty();
     assertThat(result.diagnosticHelper.getDiagnostics()).hasSize(1);
     assertThat(
-            Iterables.getOnlyElement(result.diagnosticHelper.getDiagnostics())
-                .getMessage(Locale.ENGLISH))
-        .contains("AssertionError: Cannot edit synthetic AST nodes");
+            Iterables.getOnlyElement(result.diagnosticHelper.getDiagnostics()).getMessage(ENGLISH))
+        .contains("IllegalArgumentException: Cannot edit synthetic AST nodes");
   }
 
   @Test
@@ -418,10 +414,13 @@ public class ErrorProneJavaCompilerTest {
 
   private static class CompilationResult {
     public final boolean succeeded;
+    public final String output;
     public final DiagnosticTestHelper diagnosticHelper;
 
-    public CompilationResult(boolean succeeded, DiagnosticTestHelper diagnosticHelper) {
+    public CompilationResult(
+        boolean succeeded, String output, DiagnosticTestHelper diagnosticHelper) {
       this.succeeded = succeeded;
+      this.output = output;
       this.diagnosticHelper = diagnosticHelper;
     }
   }
@@ -433,7 +432,7 @@ public class ErrorProneJavaCompilerTest {
     DiagnosticTestHelper diagnosticHelper = new DiagnosticTestHelper();
     ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
     PrintWriter printWriter = new PrintWriter(new OutputStreamWriter(outputStream, UTF_8), true);
-    ErrorProneInMemoryFileManager fileManager = new ErrorProneInMemoryFileManager();
+    JavacFileManager fileManager = FileManagers.testFileManager();
 
     List<String> args = Lists.newArrayList("-d", tempDir.getRoot().getAbsolutePath(), "-proc:none");
     args.addAll(extraArgs);
@@ -449,13 +448,9 @@ public class ErrorProneJavaCompilerTest {
             diagnosticHelper.collector,
             args,
             null,
-            fileManager.forResources(getClass(), fileNames.toArray(new String[0])));
+            forResources(getClass(), fileNames.toArray(new String[0])));
 
-    try {
-      fileManager.close();
-    } catch (IOException e) {
-      throw new UncheckedIOException(e);
-    }
-    return new CompilationResult(task.call(), diagnosticHelper);
+    return new CompilationResult(
+        task.call(), new String(outputStream.toByteArray(), UTF_8), diagnosticHelper);
   }
 }
