@@ -22,7 +22,9 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
 
-/** @author yuan@ece.toronto.edu (Ding Yuan) */
+/**
+ * @author yuan@ece.toronto.edu (Ding Yuan)
+ */
 @RunWith(JUnit4.class)
 public class EmptyCatchTest {
 
@@ -42,4 +44,27 @@ public class EmptyCatchTest {
   public void testNegativeCase() throws Exception {
     compilationHelper.addSourceFile("EmptyCatchNegativeCases.java").doTest();
   }
+
+  @Test
+  public void addTestNgTest() {
+    compilationHelper
+        .addSourceLines(
+            "org/testng/annotations/Test.java",
+            "package org.testng.annotations;",
+            "public @interface Test {",
+            "}")
+        .addSourceLines(
+            "in/SomeTest.java",
+            "import org.testng.annotations.Test;",
+            "public class SomeTest {",
+            "  @Test",
+            "  public void testNG() {",
+            "    try {",
+            "      System.err.println();",
+            "    } catch (Exception doNotCare) {",
+            "    }",
+            "  }",
+            "}")
+        .doTest();
+  }
 }
