@@ -23,7 +23,9 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
 
-/** @author eaftan@google.com (Eddie Aftandilian) */
+/**
+ * @author eaftan@google.com (Eddie Aftandilian)
+ */
 @RunWith(JUnit4.class)
 public class JUnit4TestNotRunTest {
 
@@ -31,7 +33,7 @@ public class JUnit4TestNotRunTest {
       CompilationTestHelper.newInstance(JUnit4TestNotRun.class, getClass());
 
   private final BugCheckerRefactoringTestHelper refactoringHelper =
-      BugCheckerRefactoringTestHelper.newInstance(new JUnit4TestNotRun(), getClass());
+      BugCheckerRefactoringTestHelper.newInstance(JUnit4TestNotRun.class, getClass());
 
   @Test
   public void testPositiveCase1() {
@@ -264,7 +266,6 @@ public class JUnit4TestNotRunTest {
         .doTest();
   }
 
-
   @Test
   public void hasOtherAnnotation_notATest() {
     compilationHelper
@@ -499,16 +500,12 @@ public class JUnit4TestNotRunTest {
 
   @Test
   public void testNegativeCase1() {
-    compilationHelper
-        .addSourceFile("JUnit4TestNotRunNegativeCase1.java")
-        .doTest();
+    compilationHelper.addSourceFile("JUnit4TestNotRunNegativeCase1.java").doTest();
   }
 
   @Test
   public void testNegativeCase2() {
-    compilationHelper
-        .addSourceFile("JUnit4TestNotRunNegativeCase2.java")
-        .doTest();
+    compilationHelper.addSourceFile("JUnit4TestNotRunNegativeCase2.java").doTest();
   }
 
   @Test
@@ -518,9 +515,7 @@ public class JUnit4TestNotRunTest {
 
   @Test
   public void testNegativeCase4() {
-    compilationHelper
-        .addSourceFile("JUnit4TestNotRunNegativeCase4.java")
-        .doTest();
+    compilationHelper.addSourceFile("JUnit4TestNotRunNegativeCase4.java").doTest();
   }
 
   @Test
@@ -562,6 +557,25 @@ public class JUnit4TestNotRunTest {
         .doTest();
   }
 
+  @Test
+  public void methodsWithParameters_areStillTests() {
+    compilationHelper
+        .addSourceLines(
+            "TestTheories.java",
+            "import static org.junit.Assert.fail;",
+            "import org.junit.runner.RunWith;",
+            "import org.junit.experimental.theories.Theories;",
+            "import org.junit.experimental.theories.FromDataPoints;",
+            "@RunWith(Theories.class)",
+            "public class TestTheories {",
+            "   // BUG: Diagnostic contains:",
+            "   public void testMyTheory(@FromDataPoints(\"foo\") Object foo) {",
+            "     fail();",
+            "  }",
+            "}")
+        .doTest();
+  }
+
   @Test
   public void annotationOnSuperMethod() {
     compilationHelper
