@@ -18,6 +18,7 @@ package com.google.errorprone.bugpatterns;
 
 import static com.google.errorprone.BugCheckerRefactoringTestHelper.TestMode.TEXT_MATCH;
 
+import com.google.common.collect.ImmutableList;
 import com.google.errorprone.BugCheckerRefactoringTestHelper;
 import com.google.errorprone.CompilationTestHelper;
 import org.junit.Ignore;
@@ -34,7 +35,7 @@ public class ParameterNameTest {
 
   @Test
   public void positive() {
-    BugCheckerRefactoringTestHelper.newInstance(new ParameterName(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(ParameterName.class, getClass())
         .addInputLines(
             "in/Test.java",
             "class Test {",
@@ -474,11 +475,13 @@ public class ParameterNameTest {
   }
 
   /** A test for annotated parameters across compilation boundaries. */
-  public static class AnnotatedParametersTestClass {
+  public static final class AnnotatedParametersTestClass {
     /** An annotation to apply to method parameters. */
     public @interface Annotated {}
 
     public static void target(@Annotated int foo) {}
+
+    private AnnotatedParametersTestClass() {}
   }
 
   @Test
@@ -613,7 +616,7 @@ public class ParameterNameTest {
 
   @Test
   public void varargsWrongFormat() {
-    BugCheckerRefactoringTestHelper.newInstance(new ParameterName(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(ParameterName.class, getClass())
         .addInputLines(
             "in/Test.java",
             "class Test {",
@@ -632,7 +635,33 @@ public class ParameterNameTest {
             "    foo(/* args...= */ 1, 2);",
             "  }",
             "}")
-        .doTest();
+        .doTest(TEXT_MATCH);
+  }
+
+  @Test
+  public void varargsTrailing() {
+    BugCheckerRefactoringTestHelper.newInstance(ParameterName.class, getClass())
+        .addInputLines(
+            "in/Test.java",
+            "class Test {",
+            "  void foo(int... args) {}",
+            "",
+            "  void bar() {",
+            "    foo(1, /* foo= */ 2);",
+            "    foo(1, /* foo...= */ 2);",
+            "  }",
+            "}")
+        .addOutputLines(
+            "out/Test.java",
+            "class Test {",
+            "  void foo(int... args) {}",
+            "",
+            "  void bar() {",
+            "    foo(1, /* foo */ 2);",
+            "    foo(1, /* foo... */ 2);",
+            "  }",
+            "}")
+        .doTest(TEXT_MATCH);
   }
 
   @Test
@@ -684,8 +713,10 @@ public class ParameterNameTest {
   }
 
   /** A test input for separate compilation. */
-  public static class Holder {
+  public static final class Holder {
     public static void varargsMethod(int... values) {}
+
+    private Holder() {}
   }
 
   /** Regression test for b/147344912. */
@@ -703,4 +734,42 @@ public class ParameterNameTest {
         .withClasspath(Holder.class, ParameterNameTest.class)
         .doTest();
   }
+
+  @Test
+  public void exemptPackage() {
+    CompilationTestHelper.newInstance(ParameterName.class, getClass())
+        .addSourceLines(
+            "test/a/A.java",
+            "package test.a;",
+            "public class A {",
+            "  public static void f(int value) {}",
+            "}")
+        .addSourceLines(
+            "test/b/nested/B.java",
+            "package test.b.nested;",
+            "public class B {",
+            "  public static void f(int value) {}",
+            "}")
+        .addSourceLines(
+            "test/c/C.java",
+            "package test.c;",
+            "public class C {",
+            "  public static void f(int value) {}",
+            "}")
+        .addSourceLines(
+            "Test.java",
+            "import test.a.A;",
+            "import test.b.nested.B;",
+            "import test.c.C;",
+            "class Test {",
+            "  void f() {",
+            "    A.f(/* typo= */ 1);",
+            "    B.f(/* typo= */ 1);",
+            "    // BUG: Diagnostic contains: 'C.f(/* value= */ 1);'",
+            "    C.f(/* typo= */ 1);",
+            "  }",
+            "}")
+        .setArgs(ImmutableList.of("-XepOpt:ParameterName:exemptPackagePrefixes=test.a,test.b"))
+        .doTest();
+  }
 }
