@@ -16,13 +16,14 @@ package com.google.errorprone.bugpatterns;
 
 import com.google.common.collect.ImmutableList;
 import com.google.errorprone.CompilationTestHelper;
+import org.junit.Ignore;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
 
-import org.junit.Ignore;
-
-/** @author flx@google.com (Felix Berger) */
+/**
+ * @author flx@google.com (Felix Berger)
+ */
 @RunWith(JUnit4.class)
 @Ignore("b/130670448")
 public final class ProtoFieldNullComparisonTest {
@@ -98,28 +99,6 @@ public final class ProtoFieldNullComparisonTest {
         .doTest();
   }
 
-  @Test
-  public void intermediateVariable_disabled() {
-    compilationHelper
-        .addSourceLines(
-            "Test.java",
-            "import com.google.errorprone.bugpatterns.proto.ProtoTest.TestProtoMessage;",
-            "import com.google.errorprone.bugpatterns.proto.ProtoTest.TestFieldProtoMessage;",
-            "import java.util.List;",
-            "class Test {",
-            "  void test() {",
-            "    TestProtoMessage message = TestProtoMessage.newBuilder().build();",
-            "    TestFieldProtoMessage field = message.getMessage();",
-            "    List<TestFieldProtoMessage> fields = message.getMultiFieldList();",
-            "    if (field != null) {}",
-            "    if (fields != null) {}",
-            "  }",
-            "}")
-        .setArgs(
-            ImmutableList.of("-XepOpt:ProtoFieldNullComparison:TrackServerProtoAssignments=false"))
-        .doTest();
-  }
-
   @Test
   public void negativeCases() {
     compilationHelper
@@ -283,6 +262,7 @@ public final class ProtoFieldNullComparisonTest {
             " return extensionList.build();",
             "}",
             "}")
+        .addModules("jdk.compiler/com.sun.tools.javac.code")
         .doTest();
   }
 
@@ -347,6 +327,7 @@ public final class ProtoFieldNullComparisonTest {
             "Test.java",
             "import static com.google.common.truth.Truth.assertThat;",
             "import static org.junit.Assert.assertNotNull;",
+            "import com.google.common.truth.extensions.proto.ProtoTruth;",
             "import com.google.errorprone.bugpatterns.proto.ProtoTest.TestProtoMessage;",
             "import com.google.errorprone.bugpatterns.proto.ProtoTest.TestFieldProtoMessage;",
             "import java.util.List;",
@@ -358,6 +339,8 @@ public final class ProtoFieldNullComparisonTest {
             "    assertNotNull(\"Message\", message.getMessage());",
             "    // BUG: Diagnostic contains: assertThat(message.hasMessage()).isTrue()",
             "    assertThat(message.getMessage()).isNotNull();",
+            "    // BUG: Diagnostic contains: assertThat(message.hasMessage()).isTrue()",
+            "    ProtoTruth.assertThat(message.getMessage()).isNotNull();",
             "  }",
             "}")
         .setArgs(ImmutableList.of("-XepOpt:ProtoFieldNullComparison:MatchTestAssertions"))
