@@ -21,11 +21,13 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
 
-/** @author awturner@google.com (Andy Turner) */
+/**
+ * @author awturner@google.com (Andy Turner)
+ */
 @RunWith(JUnit4.class)
 public class UnnecessaryBoxedAssignmentTest {
   private final BugCheckerRefactoringTestHelper helper =
-      BugCheckerRefactoringTestHelper.newInstance(new UnnecessaryBoxedAssignment(), getClass());
+      BugCheckerRefactoringTestHelper.newInstance(UnnecessaryBoxedAssignment.class, getClass());
 
   @Test
   public void testCases() {
