@@ -16,13 +16,17 @@
 
 package com.google.errorprone.bugpatterns.nullness;
 
+import static com.google.errorprone.BugCheckerRefactoringTestHelper.TestMode.TEXT_MATCH;
+
 import com.google.errorprone.BugCheckerRefactoringTestHelper;
 import com.google.errorprone.CompilationTestHelper;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
 
-/** @author kmb@google.com (Kevin Bierhoff) */
+/**
+ * @author kmb@google.com (Kevin Bierhoff)
+ */
 @RunWith(JUnit4.class)
 public class FieldMissingNullableTest {
 
@@ -58,6 +62,26 @@ public class FieldMissingNullableTest {
         .doTest();
   }
 
+  @Test
+  public void testAssignmentInsideIfNull() {
+    createCompilationTestHelper()
+        .addSourceLines(
+            "com/google/errorprone/bugpatterns/nullness/FieldMissingNullTest.java",
+            "package com.google.errorprone.bugpatterns.nullness;",
+            "public class FieldMissingNullTest {",
+            "  private String message;",
+            "  public void setMessage(String message) {",
+            "    if (message == null) {",
+            "      // BUG: Diagnostic contains: @Nullable",
+            "      this.message = message;",
+            "    } else {",
+            "      this.message = \"hello\";",
+            "    }",
+            "  }",
+            "}")
+        .doTest();
+  }
+
   @Test
   public void testMaybeNullAssignment() {
     createCompilationTestHelper()
@@ -74,134 +98,6 @@ public class FieldMissingNullableTest {
         .doTest();
   }
 
-  @Test
-  public void testAssignNullableMethodResult() {
-    createCompilationTestHelper()
-        .addSourceLines(
-            "com/google/errorprone/bugpatterns/nullness/NullableMethodCallTest.java",
-            "package com.google.errorprone.bugpatterns.nullness;",
-            "import javax.annotation.Nullable;",
-            "public class NullableMethodCallTest {",
-            "  private String message = \"hello\";",
-            "  public void setMessage(int x) {",
-            "    // BUG: Diagnostic contains: @Nullable",
-            "    message = toSignString(x);",
-            "  }",
-            "",
-            "  @Nullable",
-            "  private String toSignString(int x) {",
-            "    return x < 0 ? \"negative\" : \"positive\";",
-            "  }",
-            "}")
-        .doTest();
-  }
-
-  @Test
-  public void testAssignNullableMethodCall_alternativeAnnotation() {
-    createCompilationTestHelper()
-        .addSourceLines(
-            "com/google/anno/my/Nullable.java",
-            "package com.google.anno.my;",
-            "public @interface Nullable {}")
-        .addSourceLines(
-            "com/google/errorprone/bugpatterns/nullness/NullableMethodCallTest.java",
-            "package com.google.errorprone.bugpatterns.nullness;",
-            "public class NullableMethodCallTest {",
-            "  private String message = \"hello\";",
-            "  public void setMessage(int x) {",
-            "    // BUG: Diagnostic contains: @Nullable",
-            "    message = toSignString(x);",
-            "  }",
-            "",
-            "  @com.google.anno.my.Nullable",
-            "  private String toSignString(int x) {",
-            "    return x < 0 ? \"negative\" : \"positive\";",
-            "  }",
-            "}")
-        .doTest();
-  }
-
-  @Test
-  public void testAssignNullableMethodCall_typeAnnotation() {
-    createCompilationTestHelper()
-        .addSourceLines(
-            "com/google/anno/my/Nullable.java",
-            "package com.google.anno.my;",
-            "import java.lang.annotation.ElementType;",
-            "import java.lang.annotation.Target;",
-            "@Target({ElementType.TYPE_USE})",
-            "public @interface Nullable {}")
-        .addSourceLines(
-            "com/google/errorprone/bugpatterns/nullness/NullableMethodCallTest.java",
-            "package com.google.errorprone.bugpatterns.nullness;",
-            "public class NullableMethodCallTest {",
-            "  private String message = \"hello\";",
-            "  public void setMessage(int x) {",
-            "    // BUG: Diagnostic contains: @Nullable",
-            "    message = toSignString(x);",
-            "  }",
-            "",
-            "  @com.google.anno.my.Nullable",
-            "  private String toSignString(int x) {",
-            "    return x < 0 ? \"negative\" : \"positive\";",
-            "  }",
-            "}")
-        .doTest();
-  }
-
-  @Test
-  public void testAssignNullableField() {
-    createCompilationTestHelper()
-        .addSourceLines(
-            "com/google/errorprone/bugpatterns/nullness/NullableFieldTest.java",
-            "package com.google.errorprone.bugpatterns.nullness;",
-            "import javax.annotation.Nullable;",
-            "public class NullableFieldTest {",
-            "  @Nullable private String message;",
-            "  private String previous = \"\";",
-            "  public void setMessage(String message) {",
-            "    // BUG: Diagnostic contains: @Nullable",
-            "    previous = this.message;",
-            "    this.message = message;",
-            "  }",
-            "}")
-        .doTest();
-  }
-
-  @Test
-  public void testAssignNullableParameter() {
-    createCompilationTestHelper()
-        .addSourceLines(
-            "com/google/errorprone/bugpatterns/nullness/NullableParameterTest.java",
-            "package com.google.errorprone.bugpatterns.nullness;",
-            "import javax.annotation.Nullable;",
-            "public class NullableParameterTest {",
-            "  private String message = \"hello\";",
-            "  public void apply(@Nullable String message) {",
-            "    // BUG: Diagnostic contains: @Nullable",
-            "    this.message = message;",
-            "  }",
-            "}")
-        .doTest();
-  }
-
-  @Test
-  public void testInitializeWithNullableParameter() {
-    createCompilationTestHelper()
-        .addSourceLines(
-            "com/google/errorprone/bugpatterns/nullness/NullableParameterTest.java",
-            "package com.google.errorprone.bugpatterns.nullness;",
-            "import javax.annotation.Nullable;",
-            "public class NullableParameterTest {",
-            "  private final String message;",
-            "  public NullableParameterTest(@Nullable String message) {",
-            "    // BUG: Diagnostic contains: @Nullable",
-            "    this.message = message;",
-            "  }",
-            "}")
-        .doTest();
-  }
-
   @Test
   public void testNullInitializer() {
     createCompilationTestHelper()
@@ -234,6 +130,56 @@ public class FieldMissingNullableTest {
         .doTest();
   }
 
+  @Test
+  public void testComparisonToNull() {
+    createAggressiveCompilationTestHelper()
+        .addSourceLines(
+            "com/google/errorprone/bugpatterns/nullness/FieldMissingNullTest.java",
+            "package com.google.errorprone.bugpatterns.nullness;",
+            "public class FieldMissingNullTest {",
+            "  private String message;",
+            "  public void reset() {",
+            "    // BUG: Diagnostic contains: @Nullable",
+            "    if (message != null) {",
+            "    }",
+            "  }",
+            "}")
+        .doTest();
+  }
+
+  @Test
+  public void testComparisonToNullOnOtherInstance() {
+    createAggressiveCompilationTestHelper()
+        .addSourceLines(
+            "com/google/errorprone/bugpatterns/nullness/FieldMissingNullTest.java",
+            "package com.google.errorprone.bugpatterns.nullness;",
+            "public class FieldMissingNullTest {",
+            "  private String message;",
+            "  public void reset(FieldMissingNullTest other) {",
+            "    // BUG: Diagnostic contains: @Nullable",
+            "    if (other.message != null) {",
+            "    }",
+            "  }",
+            "}")
+        .doTest();
+  }
+
+  @Test
+  public void testNegativeCases_comparisonToNullConservative() {
+    createCompilationTestHelper()
+        .addSourceLines(
+            "com/google/errorprone/bugpatterns/nullness/FieldMissingNullTest.java",
+            "package com.google.errorprone.bugpatterns.nullness;",
+            "public class FieldMissingNullTest {",
+            "  private String message;",
+            "  public void reset() {",
+            "    if (message != null) {",
+            "    }",
+            "  }",
+            "}")
+        .doTest();
+  }
+
   @Test
   public void testNegativeCases_alreadyAnnotated() {
     createCompilationTestHelper()
@@ -273,6 +219,40 @@ public class FieldMissingNullableTest {
         .doTest();
   }
 
+  @Test
+  public void testNegativeCases_alreadyAnnotatedMonotonic() {
+    createCompilationTestHelper()
+        .addSourceLines(
+            "com/google/errorprone/bugpatterns/nullness/FieldMissingNullTest.java",
+            "package com.google.errorprone.bugpatterns.nullness;",
+            "import org.checkerframework.checker.nullness.qual.MonotonicNonNull;",
+            "public class FieldMissingNullTest {",
+            "  private @MonotonicNonNull String message;",
+            "  public void reset() {",
+            "    if (message != null) {",
+            "    }",
+            "  }",
+            "}")
+        .doTest();
+  }
+
+  @Test
+  public void testNegativeCases_alreadyTypeAnnotatedInnerClass() {
+    createCompilationTestHelper()
+        .addSourceLines(
+            "com/google/errorprone/bugpatterns/nullness/FieldMissingNullTest.java",
+            "package com.google.errorprone.bugpatterns.nullness;",
+            "import org.checkerframework.checker.nullness.qual.Nullable;",
+            "public class FieldMissingNullTest {",
+            "  class Inner {}",
+            "  @Nullable Inner message;",
+            "  public void reset() {",
+            "    this.message = null;",
+            "  }",
+            "}")
+        .doTest();
+  }
+
   @Test
   public void testNegativeCases_initializeWithNonNullLiteral() {
     createCompilationTestHelper()
@@ -444,7 +424,25 @@ public class FieldMissingNullableTest {
             "  @Nullable private final Object obj2 = null;",
             "  @interface Nullable {}",
             "}")
-        .doTest();
+        .doTest(TEXT_MATCH);
+  }
+
+  @Test
+  public void testAnnotationInsertedAfterModifiers() {
+    createRefactoringTestHelper()
+        .addInputLines(
+            "in/Test.java",
+            "import org.checkerframework.checker.nullness.qual.Nullable;",
+            "class T {",
+            "  private final Object obj1 = null;",
+            "}")
+        .addOutputLines(
+            "out/Test.java",
+            "import org.checkerframework.checker.nullness.qual.Nullable;",
+            "class T {",
+            "  private final @Nullable Object obj1 = null;",
+            "}")
+        .doTest(TEXT_MATCH);
   }
 
   @Test
@@ -469,7 +467,11 @@ public class FieldMissingNullableTest {
     return CompilationTestHelper.newInstance(FieldMissingNullable.class, getClass());
   }
 
+  private CompilationTestHelper createAggressiveCompilationTestHelper() {
+    return createCompilationTestHelper().setArgs("-XepOpt:Nullness:Conservative=false");
+  }
+
   private BugCheckerRefactoringTestHelper createRefactoringTestHelper() {
-    return BugCheckerRefactoringTestHelper.newInstance(new FieldMissingNullable(), getClass());
+    return BugCheckerRefactoringTestHelper.newInstance(FieldMissingNullable.class, getClass());
   }
 }
