@@ -1043,11 +1043,8 @@ public class GuardedByCheckerTest {
         .doTest();
   }
 
-  // Test that the contents of try-with-resources block are ignored (for now), but the catch and
-  // finally blocks are checked.
-  // TODO(cushon): support try-with-resources block.
   @Test
-  public void tryWithResourcesAreNotFullyUnsupported() {
+  public void tryWithResources_resourceVariables() {
     compilationHelper
         .addSourceLines(
             "threadsafety/Test.java",
@@ -1060,7 +1057,8 @@ public class GuardedByCheckerTest {
             "  int x;",
             "  void m(AutoCloseable c) throws Exception {",
             "    try (AutoCloseable unused = c) {",
-            "      x++;  // should be an error!",
+            "      // BUG: Diagnostic contains:",
+            "      x++;",
             "    } catch (Exception e) {",
             "      // BUG: Diagnostic contains:",
             "      // should be guarded by 'this.lock'",
@@ -1670,10 +1668,12 @@ public class GuardedByCheckerTest {
             "  }",
             "  public void m(Baz b) {",
             "    synchronized (mu) {",
+            "      // BUG: Diagnostic contains: 'mu', which could not be resolved",
             "      b.x++;",
             "    }",
             "  }",
             "}")
+        .setArgs("-XepOpt:GuardedByChecker:reportMissingGuards=true")
         .doTest();
   }
 
@@ -1748,4 +1748,155 @@ public class GuardedByCheckerTest {
             "}")
         .doTest();
   }
+
+  @Test
+  public void testMissingGuard() {
+    compilationHelper
+        .addSourceLines(
+            "threadsafety/Lib.java",
+            "package threadsafety;",
+            "import javax.annotation.concurrent.GuardedBy;",
+            "@SuppressWarnings(\"GuardedBy\")",
+            "class Lib {",
+            "  @GuardedBy(\"lock\")",
+            "  public void doSomething() {}",
+            "}")
+        .addSourceLines(
+            "threadsafety/Test.java",
+            "package threadsafety;",
+            "class Test {",
+            "  void m(Lib lib) {",
+            "    // BUG: Diagnostic contains: 'lock', which could not be resolved",
+            "    lib.doSomething();",
+            "  }",
+            "}")
+        .setArgs("-XepOpt:GuardedByChecker:reportMissingGuards=true")
+        .doTest();
+  }
+
+  @Test
+  public void parameterGuard() {
+    compilationHelper
+        .addSourceLines(
+            "threadsafety/Test.java",
+            "import javax.annotation.concurrent.GuardedBy;",
+            "class Work {",
+            "  final Object lock = new Object();",
+            "  Object getLock() {",
+            "    return lock;",
+            "  }",
+            "  @GuardedBy(\"getLock()\")",
+            "  void workStarted() {",
+            "  }",
+            "}",
+            "class Worker {",
+            "  @GuardedBy(\"work.getLock()\")",
+            "  void f(Work work) {",
+            "    work.workStarted(); // ok",
+            "  }",
+            "  // BUG: Diagnostic contains: could not resolve guard",
+            "  @GuardedBy(\"work2.getLock()\") void g() {}",
+            "  @GuardedBy(\"a.getLock()\")",
+            "  void g(Work a, Work b) {",
+            "    a.workStarted(); // ok",
+            "    // BUG: Diagnostic contains: should be guarded by 'b.getLock()'; instead found:"
+                + " 'a.getLock()'",
+            "    b.workStarted();",
+            "  }",
+            "}",
+            "abstract class Test {",
+            "  abstract Work getWork();",
+            "  void t(Worker worker, Work work) {",
+            "    synchronized(work.getLock()) {",
+            "      worker.f(work);",
+            "    }",
+            "    synchronized(getWork().getLock()) {",
+            "      // BUG: Diagnostic contains: guarded by 'work.getLock()'",
+            "      worker.f(getWork());",
+            "    }",
+            "    // BUG: Diagnostic contains: guarded by 'work.getLock()'",
+            "    worker.f(work);",
+            "  }",
+            "}")
+        .setArgs("-XepOpt:GuardedByChecker:reportMissingGuards=true")
+        .doTest();
+  }
+
+  @Test
+  public void parameterGuardNegative() {
+    compilationHelper
+        .addSourceLines(
+            "threadsafety/Test.java",
+            "import javax.annotation.concurrent.GuardedBy;",
+            "class Work {",
+            "  final Object lock = new Object();",
+            "  Object getLock() {",
+            "    return lock;",
+            "  }",
+            "}",
+            "class Worker {",
+            "  @GuardedBy(\"work.getLock()\")",
+            "  void f(Work work) {",
+            "  }",
+            "}",
+            "class Test {",
+            "  void t(Worker worker, Work work) {",
+            "    synchronized(work.getLock()) {",
+            "      worker.f(work);",
+            "    }",
+            "  }",
+            "}")
+        .setArgs("-XepOpt:GuardedByChecker:reportMissingGuards=true")
+        .doTest();
+  }
+
+  @Test
+  public void parameterGuardNegativeSimpleName() {
+    compilationHelper
+        .addSourceLines(
+            "threadsafety/Test.java",
+            "import javax.annotation.concurrent.GuardedBy;",
+            "class Work {",
+            "  final Object lock = new Object();",
+            "  Object getLock() {",
+            "    return lock;",
+            "  }",
+            "}",
+            "class Worker {",
+            "  @GuardedBy(\"work.getLock()\")",
+            "  void f(Work work) {",
+            "  }",
+            "  void g() {",
+            "    Work work = new Work();",
+            "    synchronized(work.getLock()) {",
+            "      f(work);",
+            "    }",
+            "  }",
+            "}")
+        .setArgs("-XepOpt:GuardedByChecker:reportMissingGuards=true")
+        .doTest();
+  }
+
+  @Test
+  public void varargsArity() {
+    compilationHelper
+        .addSourceLines(
+            "threadsafety/Test.java",
+            "import javax.annotation.concurrent.GuardedBy;",
+            "class Test {",
+            "  @GuardedBy(\"xs.toString()\")",
+            "  void f(int x, Object... xs) {",
+            "  }",
+            "  void g() {",
+            "    Object[] xs = null;",
+            "    synchronized(xs.toString()) {",
+            "      f(0, xs);",
+            "    }",
+            "    // BUG: Diagnostic contains:",
+            "    f(0);",
+            "  }",
+            "}")
+        .setArgs("-XepOpt:GuardedByChecker:reportMissingGuards=true")
+        .doTest();
+  }
 }
