@@ -23,7 +23,6 @@ import static com.google.errorprone.matchers.Description.NO_MATCH;
 import static com.google.errorprone.matchers.Matchers.isSameType;
 import static com.google.errorprone.matchers.Matchers.staticMethod;
 import static java.lang.annotation.RetentionPolicy.RUNTIME;
-import static org.junit.Assume.assumeTrue;
 
 import com.google.common.base.Verify;
 import com.google.common.collect.ImmutableList;
@@ -34,6 +33,7 @@ import com.google.errorprone.BugPattern;
 import com.google.errorprone.CompilationTestHelper;
 import com.google.errorprone.VisitorState;
 import com.google.errorprone.bugpatterns.BugChecker;
+import com.google.errorprone.bugpatterns.BugChecker.AnnotationTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.ClassTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.LiteralTreeMatcher;
 import com.google.errorprone.bugpatterns.BugChecker.MethodInvocationTreeMatcher;
@@ -45,10 +45,12 @@ import com.google.errorprone.bugpatterns.RemoveUnusedImports;
 import com.google.errorprone.matchers.Description;
 import com.google.errorprone.matchers.Matcher;
 import com.google.errorprone.util.ASTHelpers;
-import com.google.errorprone.util.RuntimeVersion;
 import com.sun.source.doctree.LinkTree;
+import com.sun.source.tree.AnnotationTree;
+import com.sun.source.tree.AssignmentTree;
 import com.sun.source.tree.ClassTree;
 import com.sun.source.tree.ExpressionTree;
+import com.sun.source.tree.IdentifierTree;
 import com.sun.source.tree.LambdaExpressionTree;
 import com.sun.source.tree.LiteralTree;
 import com.sun.source.tree.MethodInvocationTree;
@@ -73,7 +75,9 @@ import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.JUnit4;
 
-/** @author cushon@google.com (Liam Miller-Cushon) */
+/**
+ * @author cushon@google.com (Liam Miller-Cushon)
+ */
 @RunWith(JUnit4.class)
 public class SuggestedFixesTest {
 
@@ -103,7 +107,7 @@ public class SuggestedFixesTest {
       for (Modifier mod : Modifier.values()) {
         builder.put(mod.toString(), mod);
       }
-      return builder.build();
+      return builder.buildOrThrow();
     }
 
     @Override
@@ -141,7 +145,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void addAtBeginningOfLine() {
-    BugCheckerRefactoringTestHelper.newInstance(new EditModifiersChecker(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(EditModifiersChecker.class, getClass())
         .addInputLines(
             "in/Test.java",
             "import javax.annotation.Nullable;",
@@ -184,6 +188,8 @@ public class SuggestedFixesTest {
             "  @Nullable public Object three;",
             "  // BUG: Diagnostic contains: public final Object four",
             "  public Object four;",
+            "  // BUG: Diagnostic contains: public final @Nullable Object five",
+            "  public @Nullable Object five;",
             "}")
         .doTest();
   }
@@ -259,7 +265,7 @@ public class SuggestedFixesTest {
   }
 
   /** Test checker that casts returned expression. */
-  @BugPattern(name = "CastReturn", severity = ERROR, summary = "Adds casts to returned expressions")
+  @BugPattern(severity = ERROR, summary = "Adds casts to returned expressions")
   public static class CastReturn extends BugChecker implements ReturnTreeMatcher {
 
     @Override
@@ -411,7 +417,7 @@ public class SuggestedFixesTest {
   }
 
   /** A test check that adds an annotation to all return types. */
-  @BugPattern(name = "AddAnnotation", summary = "Add an annotation", severity = ERROR)
+  @BugPattern(summary = "Add an annotation", severity = ERROR)
   public static class AddAnnotation extends BugChecker implements BugChecker.MethodTreeMatcher {
     @Override
     public Description matchMethod(MethodTree tree, VisitorState state) {
@@ -425,7 +431,7 @@ public class SuggestedFixesTest {
 
     private static BugCheckerRefactoringTestHelper testHelper(
         Class<? extends SuggestedFixesTest> clazz) {
-      return BugCheckerRefactoringTestHelper.newInstance(new AddAnnotation(), clazz)
+      return BugCheckerRefactoringTestHelper.newInstance(AddAnnotation.class, clazz)
           .addInputLines(
               "in/some/pkg/SomeAnnotation.java",
               "package some.pkg;",
@@ -567,10 +573,7 @@ public class SuggestedFixesTest {
   }
 
   /** A test check that replaces all methods' return types with a given type. */
-  @BugPattern(
-      name = "ReplaceReturnType",
-      summary = "Change the method return type",
-      severity = ERROR)
+  @BugPattern(summary = "Change the method return type", severity = ERROR)
   public static class ReplaceReturnType extends BugChecker implements BugChecker.MethodTreeMatcher {
     private final String newReturnType;
 
@@ -599,10 +602,7 @@ public class SuggestedFixesTest {
   }
 
   /** A test check that replaces all methods' return types with a given type. */
-  @BugPattern(
-      name = "ReplaceReturnTypeString",
-      summary = "Change the method return type",
-      severity = ERROR)
+  @BugPattern(summary = "Change the method return type", severity = ERROR)
   public static class ReplaceReturnTypeString extends BugChecker
       implements BugChecker.MethodTreeMatcher {
     private final String newReturnType;
@@ -765,10 +765,7 @@ public class SuggestedFixesTest {
   }
 
   /** A test check that replaces all checkNotNulls with verifyNotNull. */
-  @BugPattern(
-      name = "ReplaceMethodInvocations",
-      summary = "Replaces checkNotNull with verifyNotNull.",
-      severity = ERROR)
+  @BugPattern(summary = "Replaces checkNotNull with verifyNotNull.", severity = ERROR)
   public static class ReplaceMethodInvocations extends BugChecker
       implements BugChecker.MethodInvocationTreeMatcher {
     private static final Matcher<ExpressionTree> CHECK_NOT_NULL =
@@ -796,7 +793,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void qualifyStaticImport_addsStaticImportAndUsesUnqualifiedName() {
-    BugCheckerRefactoringTestHelper.newInstance(new ReplaceMethodInvocations(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(ReplaceMethodInvocations.class, getClass())
         .addInputLines(
             "Test.java",
             "import static com.google.common.base.Preconditions.checkNotNull;",
@@ -819,7 +816,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void qualifyStaticImport_whenAlreadyImported_usesUnqualifiedName() {
-    BugCheckerRefactoringTestHelper.newInstance(new ReplaceMethodInvocations(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(ReplaceMethodInvocations.class, getClass())
         .addInputLines(
             "Test.java",
             "import static com.google.common.base.Preconditions.checkNotNull;",
@@ -844,14 +841,47 @@ public class SuggestedFixesTest {
   }
 
   @Test
-  public void qualifyStaticImport_whenNamesClash_usesQualifiedName() {
-    BugCheckerRefactoringTestHelper.newInstance(new ReplaceMethodInvocations(), getClass())
+  public void qualifyStaticImport_whenIdentifierNamesClash_usesQualifiedName() {
+    BugCheckerRefactoringTestHelper.newInstance(ReplaceMethodInvocations.class, getClass())
+        .addInputLines(
+            "pkg/Lib.java",
+            "package pkg;",
+            "public class Lib {",
+            "  public static void verifyNotNull(int a) {}",
+            " }")
+        .expectUnchanged()
+        .addInputLines(
+            "Test.java",
+            "import static com.google.common.base.Preconditions.checkNotNull;",
+            "import static pkg.Lib.verifyNotNull;",
+            "class Test {",
+            "  void test() {",
+            "    verifyNotNull(2);",
+            "    checkNotNull(1);",
+            "  }",
+            "}")
+        .addOutputLines(
+            "Test.java",
+            "import static com.google.common.base.Preconditions.checkNotNull;",
+            "import static pkg.Lib.verifyNotNull;",
+            "import com.google.common.base.Verify;",
+            "class Test {",
+            "  void test() {",
+            "    verifyNotNull(2);",
+            "    Verify.verifyNotNull(1);",
+            "  }",
+            "}")
+        .doTest();
+  }
+
+  @Test
+  public void qualifyStaticImport_whenMethodNamesClash_usesQualifiedName() {
+    BugCheckerRefactoringTestHelper.newInstance(ReplaceMethodInvocations.class, getClass())
         .addInputLines(
             "Test.java",
             "import static com.google.common.base.Preconditions.checkNotNull;",
             "class Test {",
             "  void test() {",
-            "    verifyNotNull(2);",
             "    checkNotNull(1);",
             "  }",
             "  void verifyNotNull(int a) {}",
@@ -862,7 +892,6 @@ public class SuggestedFixesTest {
             "import com.google.common.base.Verify;",
             "class Test {",
             "  void test() {",
-            "    verifyNotNull(2);",
             "    Verify.verifyNotNull(1);",
             "  }",
             "  void verifyNotNull(int a) {}",
@@ -871,20 +900,17 @@ public class SuggestedFixesTest {
   }
 
   /** A test check that qualifies javadoc link. */
-  @BugPattern(
-      name = "JavadocQualifier",
-      summary = "all javadoc links should be qualified",
-      severity = ERROR)
+  @BugPattern(summary = "all javadoc links should be qualified", severity = ERROR)
   public static class JavadocQualifier extends BugChecker implements BugChecker.ClassTreeMatcher {
     @Override
-    public Description matchClass(ClassTree tree, final VisitorState state) {
-      final DCTree.DCDocComment comment =
+    public Description matchClass(ClassTree tree, VisitorState state) {
+      DCTree.DCDocComment comment =
           ((JCTree.JCCompilationUnit) state.getPath().getCompilationUnit())
               .docComments.getCommentTree((JCTree) tree);
       if (comment == null) {
         return Description.NO_MATCH;
       }
-      final SuggestedFix.Builder fix = SuggestedFix.builder();
+      SuggestedFix.Builder fix = SuggestedFix.builder();
       new DocTreePathScanner<Void, Void>() {
         @Override
         public Void visitLink(LinkTree node, Void unused) {
@@ -902,7 +928,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void qualifyJavadocTest() {
-    BugCheckerRefactoringTestHelper.newInstance(new JavadocQualifier(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(JavadocQualifier.class, getClass())
         .addInputLines(
             "in/Test.java", //
             "import java.util.List;",
@@ -923,8 +949,9 @@ public class SuggestedFixesTest {
         .doTest(TEXT_MATCH);
   }
 
-  @BugPattern(name = "SuppressMe", summary = "", severity = ERROR)
-  static final class SuppressMe extends BugChecker
+  /** A {@link BugChecker} for testing. */
+  @BugPattern(summary = "", severity = ERROR)
+  public static final class SuppressMe extends BugChecker
       implements LiteralTreeMatcher, VariableTreeMatcher {
     @Override
     public Description matchLiteral(LiteralTree tree, VisitorState state) {
@@ -954,7 +981,7 @@ public class SuggestedFixesTest {
   @org.junit.Ignore("There appears to be an issue parsing lambda comments")
   public void testSuppressWarningsFix() {
     BugCheckerRefactoringTestHelper refactorTestHelper =
-        BugCheckerRefactoringTestHelper.newInstance(new SuppressMe(), getClass());
+        BugCheckerRefactoringTestHelper.newInstance(SuppressMe.class, getClass());
     refactorTestHelper
         .addInputLines(
             "in/Test.java",
@@ -998,8 +1025,9 @@ public class SuggestedFixesTest {
         .doTest();
   }
 
-  @BugPattern(name = "SuppressMeWithComment", summary = "", severity = ERROR)
-  static final class SuppressMeWithComment extends BugChecker implements LiteralTreeMatcher {
+  /** A {@link BugChecker} for testing. */
+  @BugPattern(summary = "", severity = ERROR)
+  public static final class SuppressMeWithComment extends BugChecker implements LiteralTreeMatcher {
     private final String lineComment;
 
     SuppressMeWithComment(String lineComment) {
@@ -1086,8 +1114,9 @@ public class SuggestedFixesTest {
         .doTest(BugCheckerRefactoringTestHelper.TestMode.TEXT_MATCH);
   }
 
-  @BugPattern(name = "RemoveSuppressFromMe", summary = "", severity = ERROR)
-  static final class RemoveSuppressFromMe extends BugChecker implements LiteralTreeMatcher {
+  /** A {@link BugChecker} for testing. */
+  @BugPattern(summary = "", severity = ERROR)
+  public static final class RemoveSuppressFromMe extends BugChecker implements LiteralTreeMatcher {
 
     @Override
     public Description matchLiteral(LiteralTree tree, VisitorState state) {
@@ -1100,7 +1129,7 @@ public class SuggestedFixesTest {
   @Test
   public void removeSuppressWarnings_singleWarning_removesEntireAnnotation() {
     BugCheckerRefactoringTestHelper refactorTestHelper =
-        BugCheckerRefactoringTestHelper.newInstance(new RemoveSuppressFromMe(), getClass());
+        BugCheckerRefactoringTestHelper.newInstance(RemoveSuppressFromMe.class, getClass());
     refactorTestHelper
         .addInputLines(
             "in/Test.java",
@@ -1114,7 +1143,7 @@ public class SuggestedFixesTest {
   @Test
   public void removeSuppressWarnings_twoWarning_removesWarningAndNewArray() {
     BugCheckerRefactoringTestHelper refactorTestHelper =
-        BugCheckerRefactoringTestHelper.newInstance(new RemoveSuppressFromMe(), getClass());
+        BugCheckerRefactoringTestHelper.newInstance(RemoveSuppressFromMe.class, getClass());
     refactorTestHelper
         .addInputLines(
             "in/Test.java",
@@ -1132,7 +1161,7 @@ public class SuggestedFixesTest {
   @Test
   public void removeSuppressWarnings_threeWarning_removesOnlyOneAndKeepsArray() {
     BugCheckerRefactoringTestHelper refactorTestHelper =
-        BugCheckerRefactoringTestHelper.newInstance(new RemoveSuppressFromMe(), getClass());
+        BugCheckerRefactoringTestHelper.newInstance(RemoveSuppressFromMe.class, getClass());
     refactorTestHelper
         .addInputLines(
             "in/Test.java",
@@ -1150,7 +1179,7 @@ public class SuggestedFixesTest {
   @Test
   public void removeSuppressWarnings_oneWarningInArray_removesWholeAnnotation() {
     BugCheckerRefactoringTestHelper refactorTestHelper =
-        BugCheckerRefactoringTestHelper.newInstance(new RemoveSuppressFromMe(), getClass());
+        BugCheckerRefactoringTestHelper.newInstance(RemoveSuppressFromMe.class, getClass());
     refactorTestHelper
         .addInputLines(
             "in/Test.java",
@@ -1164,7 +1193,7 @@ public class SuggestedFixesTest {
   @Test
   public void removeSuppressWarnings_withValueInit_retainsValue() {
     BugCheckerRefactoringTestHelper refactorTestHelper =
-        BugCheckerRefactoringTestHelper.newInstance(new RemoveSuppressFromMe(), getClass());
+        BugCheckerRefactoringTestHelper.newInstance(RemoveSuppressFromMe.class, getClass());
     refactorTestHelper
         .addInputLines(
             "in/Test.java",
@@ -1179,8 +1208,42 @@ public class SuggestedFixesTest {
         .doTest(TestMode.AST_MATCH);
   }
 
+  /** A {@link BugChecker} for testing. */
+  @BugPattern(name = "UpdateDoNotCallArgument", summary = "", severity = ERROR)
+  public static final class UpdateDoNotCallArgumentChecker extends BugChecker
+      implements AnnotationTreeMatcher {
+    @Override
+    public Description matchAnnotation(AnnotationTree tree, VisitorState state) {
+      SuggestedFix.Builder fixBuilder =
+          SuggestedFixes.updateAnnotationArgumentValues(
+              tree, state, "value", ImmutableList.of("\"Danger\""));
+      return describeMatch(tree, fixBuilder.build());
+    }
+  }
+
+  @Test
+  public void updateAnnotationArgumentValues_noArguments() {
+    BugCheckerRefactoringTestHelper refactorTestHelper =
+        BugCheckerRefactoringTestHelper.newInstance(
+            UpdateDoNotCallArgumentChecker.class, getClass());
+    refactorTestHelper
+        .addInputLines(
+            "in/Test.java",
+            "import com.google.errorprone.annotations.DoNotCall;",
+            "public class Test {",
+            "  @DoNotCall void m() {}",
+            "}")
+        .addOutputLines(
+            "out/Test.java",
+            "import com.google.errorprone.annotations.DoNotCall;",
+            "public class Test {",
+            "  @DoNotCall(\"Danger\") void m() {}",
+            "}")
+        .doTest(TestMode.AST_MATCH);
+  }
+
   /** A test bugchecker that deletes any field whose removal doesn't break the compilation. */
-  @BugPattern(name = "CompilesWithFixChecker", summary = "", severity = ERROR)
+  @BugPattern(summary = "", severity = ERROR)
   public static class CompilesWithFixChecker extends BugChecker implements VariableTreeMatcher {
     @Override
     public Description matchVariable(VariableTree tree, VisitorState state) {
@@ -1193,7 +1256,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void compilesWithFixTest() {
-    BugCheckerRefactoringTestHelper.newInstance(new CompilesWithFixChecker(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(CompilesWithFixChecker.class, getClass())
         .addInputLines(
             "in/Test.java",
             "class Test {",
@@ -1216,8 +1279,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void compilesWithFix_releaseFlag() {
-    assumeTrue(RuntimeVersion.isAtLeast9());
-    BugCheckerRefactoringTestHelper.newInstance(new CompilesWithFixChecker(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(CompilesWithFixChecker.class, getClass())
         .setArgs("--release", "9")
         .addInputLines(
             "in/Test.java",
@@ -1337,7 +1399,7 @@ public class SuggestedFixesTest {
   }
 
   /** Test checker that renames variables. */
-  @BugPattern(name = "RenamesVariableChecker", summary = "", severity = ERROR)
+  @BugPattern(summary = "", severity = ERROR)
   public static class RenamesVariableChecker extends BugChecker implements VariableTreeMatcher {
 
     private final String toReplace;
@@ -1535,7 +1597,7 @@ public class SuggestedFixesTest {
   }
 
   /** Test checker that removes and adds modifiers in the same fix. */
-  @BugPattern(name = "RemoveAddModifier", summary = "", severity = ERROR)
+  @BugPattern(summary = "", severity = ERROR)
   public static class RemoveAddModifier extends BugChecker implements ClassTreeMatcher {
 
     @Override
@@ -1551,17 +1613,14 @@ public class SuggestedFixesTest {
 
   @Test
   public void removeAddModifier_rangesCompatible() {
-    BugCheckerRefactoringTestHelper.newInstance(new RemoveAddModifier(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(RemoveAddModifier.class, getClass())
         .addInputLines("in/Test.java", "public class Test {}")
         .addOutputLines("out/Test.java", "abstract class Test {}")
         .doTest();
   }
 
   /** A bugchecker for testing suggested fixes. */
-  @BugPattern(
-      name = "PrefixAddImportCheck",
-      summary = "A bugchecker for testing suggested fixes.",
-      severity = ERROR)
+  @BugPattern(summary = "A bugchecker for testing suggested fixes.", severity = ERROR)
   public static class PrefixAddImportCheck extends BugChecker implements ClassTreeMatcher {
     @Override
     public Description matchClass(ClassTree tree, VisitorState state) {
@@ -1576,7 +1635,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void prefixAddImport() throws IOException {
-    BugCheckerRefactoringTestHelper.newInstance(new PrefixAddImportCheck(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(PrefixAddImportCheck.class, getClass())
         .addInputLines(
             "in/Test.java", //
             "package p;",
@@ -1598,8 +1657,9 @@ public class SuggestedFixesTest {
         .isEqualTo(URI.create("file:/com/google/Foo.java"));
   }
 
-  @BugPattern(name = "RenameMethodChecker", summary = "RenameMethodChecker", severity = ERROR)
-  private static class RenameMethodChecker extends BugChecker
+  /** A {@link BugChecker} for testing. */
+  @BugPattern(summary = "RenameMethodChecker", severity = ERROR)
+  public static class RenameMethodChecker extends BugChecker
       implements MethodInvocationTreeMatcher {
     @Override
     public Description matchMethodInvocation(MethodInvocationTree tree, VisitorState state) {
@@ -1609,7 +1669,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void renameMethodInvocation() {
-    BugCheckerRefactoringTestHelper.newInstance(new RenameMethodChecker(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(RenameMethodChecker.class, getClass())
         .addInputLines(
             "Test.java",
             "import java.util.Collections;",
@@ -1639,10 +1699,7 @@ public class SuggestedFixesTest {
    * Test checker that raises a diagnostic with the result of {@link SuggestedFixes#qualifyType} on
    * new instances.
    */
-  @BugPattern(
-      name = "QualifyTypeLocalClassChecker",
-      summary = "QualifyTypeLocalClassChecker",
-      severity = ERROR)
+  @BugPattern(summary = "QualifyTypeLocalClassChecker", severity = ERROR)
   public static class QualifyTypeLocalClassChecker extends BugChecker
       implements NewClassTreeMatcher {
 
@@ -1753,10 +1810,7 @@ public class SuggestedFixesTest {
   }
 
   /** Test checker that adds @SuppressWarnings when compilation succeeds in the current unit. */
-  @BugPattern(
-      name = "AddSuppressWarningsIfCompilationSucceedsOnlyInSameCompilationUnit",
-      summary = "",
-      severity = ERROR)
+  @BugPattern(summary = "", severity = ERROR)
   public static final class AddSuppressWarningsIfCompilationSucceedsOnlyInSameCompilationUnit
       extends BugChecker implements ClassTreeMatcher {
     @Override
@@ -1801,10 +1855,7 @@ public class SuggestedFixesTest {
   }
 
   /** Test checker that adds @SuppressWarnings when compilation succeeds in all units. */
-  @BugPattern(
-      name = "AddSuppressWarningsIfCompilationSucceedsInAllCompilationUnits",
-      summary = "",
-      severity = ERROR)
+  @BugPattern(summary = "", severity = ERROR)
   public static final class AddSuppressWarningsIfCompilationSucceedsInAllCompilationUnits
       extends BugChecker implements ClassTreeMatcher {
     @Override
@@ -1866,7 +1917,7 @@ public class SuggestedFixesTest {
 
   @Test
   public void castTree() {
-    BugCheckerRefactoringTestHelper.newInstance(new CastTreeToIntChecker(), getClass())
+    BugCheckerRefactoringTestHelper.newInstance(CastTreeToIntChecker.class, getClass())
         .addInputLines(
             "Test.java",
             "class Test {",
@@ -1889,4 +1940,79 @@ public class SuggestedFixesTest {
             "}")
         .doTest();
   }
+
+  @Test
+  public void addDuplicateImport() {
+    String firstImport = "java.time.Duration";
+    String secondImport = "java.time.Instant";
+    SuggestedFix fix =
+        SuggestedFix.builder()
+            .addImport(firstImport)
+            .addImport(secondImport)
+            .addImport(firstImport)
+            .build();
+    assertThat(fix.getImportsToAdd())
+        .containsExactly("import " + firstImport, "import " + secondImport)
+        .inOrder();
+  }
+
+  @Test
+  public void addDuplicateStaticImport() {
+    String firstImport = "java.util.concurrent.TimeUnit.MILLISECONDS";
+    String secondImport = "java.util.concurrent.TimeUnit.SECONDS";
+    SuggestedFix fix =
+        SuggestedFix.builder()
+            .addStaticImport(firstImport)
+            .addStaticImport(secondImport)
+            .addStaticImport(firstImport)
+            .build();
+    assertThat(fix.getImportsToAdd())
+        .containsExactly("import static " + firstImport, "import static " + secondImport)
+        .inOrder();
+  }
+
+  @Test
+  public void removeElement() {
+    BugCheckerRefactoringTestHelper.newInstance(RemoveAnnotationElement.class, getClass())
+        .addInputLines(
+            "Anno.java", //
+            "@interface Anno {",
+            "  int a() default 0;",
+            "  int b() default 0;",
+            "}")
+        .expectUnchanged()
+        .addInputLines(
+            "Test.java",
+            "class Test {",
+            "  @Anno(a = 1) class A {}",
+            "  @Anno(a = 1, b = 2) class B {}",
+            "  @Anno(b = 1, a = 2) class C {}",
+            "}")
+        .addOutputLines(
+            "Test.java",
+            "class Test {",
+            "  @Anno() class A {}",
+            "  @Anno(b = 2) class B {}",
+            "  @Anno(b = 1) class C {}",
+            "}")
+        .doTest();
+  }
+
+  /** Bugpattern for testing. */
+  @BugPattern(name = "RemoveAnnotationElement", summary = "", severity = ERROR)
+  public static final class RemoveAnnotationElement extends BugChecker
+      implements AnnotationTreeMatcher {
+    @Override
+    public Description matchAnnotation(AnnotationTree tree, VisitorState state) {
+      return tree.getArguments().stream()
+          .filter(
+              t ->
+                  ((IdentifierTree) ((AssignmentTree) t).getVariable())
+                      .getName()
+                      .contentEquals("a"))
+          .findFirst()
+          .map(t -> describeMatch(t, SuggestedFixes.removeElement(t, tree.getArguments(), state)))
+          .orElse(NO_MATCH);
+    }
+  }
 }
